"""函数分析器测试

测试函数分析器的功能，包括函数签名分析、参数检测、类型注解处理等。
"""

import pytest
import inspect
from typing import Optional, List
from unittest.mock import Mock
from ncatbot.core.event import BaseMessageEvent

from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.analyzer.func_analyzer import (
    FuncAnalyser, get_subclass_recursive, FuncSpec
)


class TestGetSubclassRecursive:
    """递归获取子类测试"""
    
    def test_get_subclass_simple(self):
        """测试简单类的子类获取"""
        class BaseClass:
            pass
        
        class ChildClass(BaseClass):
            pass
        
        subclasses = get_subclass_recursive(BaseClass)
        assert BaseClass in subclasses
        assert ChildClass in subclasses
        assert len(subclasses) >= 2
    
    def test_get_subclass_nested(self):
        """测试嵌套继承的子类获取"""
        class GrandParent:
            pass
        
        class Parent(GrandParent):
            pass
        
        class Child(Parent):
            pass
        
        class GrandChild(Child):
            pass
        
        subclasses = get_subclass_recursive(GrandParent)
        assert GrandParent in subclasses
        assert Parent in subclasses
        assert Child in subclasses
        assert GrandChild in subclasses
    
    def test_get_subclass_no_children(self):
        """测试没有子类的类"""
        class LonelyClass:
            pass
        
        subclasses = get_subclass_recursive(LonelyClass)
        assert subclasses == [LonelyClass]
    
    def test_get_subclass_multiple_inheritance(self):
        """测试多重继承的子类获取"""
        class MixinA:
            pass
        
        class MixinB:
            pass
        
        class Combined(MixinA, MixinB):
            pass
        
        # 测试从不同基类开始
        subclasses_a = get_subclass_recursive(MixinA)
        subclasses_b = get_subclass_recursive(MixinB)
        
        assert MixinA in subclasses_a
        assert Combined in subclasses_a
        assert MixinB in subclasses_b
        assert Combined in subclasses_b


class TestFuncDescriptor:
    """函数描述符测试"""
    
    def test_descriptor_creation_simple(self):
        """测试简单函数的描述符创建"""
        def simple_func():
            """简单函数"""
            pass
        
        descriptor = FuncSpec(simple_func)
        
        assert descriptor.func is simple_func
        assert descriptor.func_name == "simple_func"
        assert descriptor.func_module == simple_func.__module__
        assert descriptor.func_qualname == simple_func.__qualname__
        assert isinstance(descriptor.signature, inspect.Signature)
        assert len(descriptor.param_list) == 0
        assert len(descriptor.param_names) == 0
        assert len(descriptor.param_annotations) == 0
    
    def test_descriptor_with_parameters(self):
        """测试带参数函数的描述符"""
        def func_with_params(event, name: str, age: int = 18, verbose: bool = False):
            """带参数的函数"""
            pass
        
        descriptor = FuncSpec(func_with_params)
        
        assert descriptor.func_name == "func_with_params"
        assert len(descriptor.param_list) == 4
        assert descriptor.param_names == ["event", "name", "age", "verbose"]
        
        # 检查参数注解
        annotations = descriptor.param_annotations
        assert len(annotations) == 4
        assert annotations[1] is str  # name: str
        assert annotations[2] is int  # age: int
        assert annotations[3] is bool # verbose: bool
    
    def test_descriptor_with_aliases(self):
        """测试带别名的函数描述符"""
        def func_with_alias():
            pass
        
        func_with_alias.__aliases__ = ["alias1", "alias2"]
        
        descriptor = FuncSpec(func_with_alias)
        assert descriptor.aliases == ["alias1", "alias2"]
    
    def test_descriptor_no_aliases(self):
        """测试没有别名的函数描述符"""
        def func_no_alias():
            pass
        
        descriptor = FuncSpec(func_no_alias)
        assert descriptor.aliases == []
    
    def test_descriptor_complex_signature(self):
        """测试复杂签名的函数描述符"""
        def complex_func(event, *args, name: str = "default", **kwargs) -> Optional[str]:
            """复杂函数签名"""
            pass
        
        descriptor = FuncSpec(complex_func)
        
        assert descriptor.func_name == "complex_func"
        assert len(descriptor.param_list) >= 4  # event, *args, name, **kwargs
        
        # 检查参数名称
        param_names = descriptor.param_names
        assert "event" in param_names
        assert "args" in param_names
        assert "name" in param_names
        assert "kwargs" in param_names


class TestFuncAnalyser:
    """函数分析器测试"""
    
    def test_analyzer_creation(self):
        """测试分析器创建"""
        def test_func():
            pass
        
        analyzer = FuncAnalyser(test_func)
        assert analyzer.func_descriptor is not None
        assert analyzer.func_descriptor.func is test_func
    
    def test_analyzer_simple_function(self):
        """测试分析简单函数"""
        def simple_func(event: BaseMessageEvent):
            """简单事件处理函数"""
            return "simple"
        
        analyzer = FuncAnalyser(simple_func)
        
        # 检查描述符
        descriptor = analyzer.func_descriptor
        assert descriptor.func_name == "simple_func"
        assert len(descriptor.param_list) == 1
        assert descriptor.param_names[0] == "event"
    
    def test_analyzer_function_with_params(self):
        """测试分析带参数的函数"""
        def param_func(event, name: str, age: int = 25, verbose: bool = False):
            """带参数的函数"""
            return f"Hello {name}, age {age}"
        
        analyzer = FuncAnalyser(param_func)
        descriptor = analyzer.func_descriptor
        
        # 检查参数数量和类型
        assert len(descriptor.param_list) == 4
        assert descriptor.param_names == ["event", "name", "age", "verbose"]
        
        # 检查默认值
        params = descriptor.param_list
        assert params[0].default == inspect.Parameter.empty  # event 无默认值
        assert params[1].default == inspect.Parameter.empty  # name 无默认值
        assert params[2].default == 25  # age 有默认值
        assert params[3].default == False  # verbose 有默认值
    
    def test_analyzer_function_with_annotations(self):
        """测试分析带类型注解的函数"""
        def annotated_func(event, data: List[str], count: Optional[int] = None) -> str:
            """带类型注解的函数"""
            return "annotated"
        
        analyzer = FuncAnalyser(annotated_func)
        descriptor = analyzer.func_descriptor
        
        # 检查类型注解
        annotations = descriptor.param_annotations
        assert len(annotations) == 3
        assert annotations[1] == List[str]  # data: List[str]
        assert annotations[2] == Optional[int]  # count: Optional[int]
        
        # 检查返回类型注解
        assert descriptor.signature.return_annotation == str
    
    def test_analyzer_function_with_varargs(self):
        """测试分析带可变参数的函数"""
        def varargs_func(event, *args, **kwargs):
            """带可变参数的函数"""
            return "varargs"
        
        analyzer = FuncAnalyser(varargs_func)
        descriptor = analyzer.func_descriptor
        
        # 检查参数类型
        params = descriptor.param_list
        assert len(params) == 3
        
        # 检查参数种类
        assert params[0].kind == inspect.Parameter.POSITIONAL_OR_KEYWORD  # event
        assert params[1].kind == inspect.Parameter.VAR_POSITIONAL  # *args
        assert params[2].kind == inspect.Parameter.VAR_KEYWORD  # **kwargs
    
    def test_analyzer_function_with_keyword_only(self):
        """测试分析带仅关键字参数的函数"""
        def kwonly_func(event, name: str, *, force: bool = False, debug: bool = False):
            """带仅关键字参数的函数"""
            return "kwonly"
        
        analyzer = FuncAnalyser(kwonly_func)
        descriptor = analyzer.func_descriptor
        
        # 检查参数
        params = descriptor.param_list
        assert len(params) == 4
        
        # 检查仅关键字参数
        force_param = next(p for p in params if p.name == "force")
        debug_param = next(p for p in params if p.name == "debug")
        
        assert force_param.kind == inspect.Parameter.KEYWORD_ONLY
        assert debug_param.kind == inspect.Parameter.KEYWORD_ONLY
        assert force_param.default == False
        assert debug_param.default == False


class TestFuncAnalyserAnalyze:
    """函数分析器analyze方法测试"""
    
    def test_analyze_method_exists(self):
        """测试analyze方法存在"""
        def test_func(event: BaseMessageEvent):
            pass
        
        analyzer = FuncAnalyser(test_func)
        assert hasattr(analyzer, 'analyze')
        assert callable(analyzer.analyze)
    
    # @pytest.mark.skip(reason="需要实际的analyze方法实现")
    def test_analyze_returns_command_spec(self):
        """测试analyze方法返回CommandSpec"""
        def test_func(event: BaseMessageEvent, name: str = "default"):
            """测试函数"""
            return "test"
        
        analyzer = FuncAnalyser(test_func)
        result = analyzer.analyze()
        
        # 根据实际实现调整断言
        assert result is not None
        # assert isinstance(result, CommandSpec)
    
    # @pytest.mark.skip(reason="需要实际的analyze方法实现")
    def test_analyze_complex_function(self):
        """测试分析复杂函数"""
        def complex_func(event: BaseMessageEvent, 
                        target: str,
                        data: str,
                        count: int = 1,
                        force: bool = False,
                        debug: bool = False) -> Optional[str]:
            """复杂函数示例"""
            return "complex"
        
        analyzer = FuncAnalyser(complex_func)
        result = analyzer.analyze()
        
        # 验证分析结果
        assert result is not None


class TestFuncAnalyserEdgeCases:
    """函数分析器边界情况测试"""
    
    def test_analyzer_lambda_function(self):
        """测试分析lambda函数"""
        lambda_func = lambda event: "lambda result"
        
        analyzer = FuncAnalyser(lambda_func)
        descriptor = analyzer.func_descriptor
        
        assert descriptor.func is lambda_func
        assert descriptor.func_name == "<lambda>"
        assert len(descriptor.param_list) == 1
    
    def test_analyzer_nested_function(self):
        """测试分析嵌套函数"""
        def outer_func():
            def inner_func(event, data: str):
                return f"inner: {data}"
            return inner_func
        
        inner = outer_func()
        analyzer = FuncAnalyser(inner)
        descriptor = analyzer.func_descriptor
        
        assert descriptor.func is inner
        assert descriptor.func_name == "inner_func"
        assert len(descriptor.param_list) == 2
    
    def test_analyzer_static_method(self):
        """测试分析静态方法"""
        class TestClass:
            @staticmethod
            def static_method(event, data: str):
                return f"static: {data}"
        
        analyzer = FuncAnalyser(TestClass.static_method)
        descriptor = analyzer.func_descriptor
        
        assert descriptor.func is TestClass.static_method
        assert descriptor.func_name == "static_method"
        assert len(descriptor.param_list) == 2  # event, data
    
    def test_analyzer_function_with_docstring(self):
        """测试分析带文档字符串的函数"""
        def documented_func(event: BaseMessageEvent):
            """这是一个有文档的函数
            
            Args:
                event: 事件参数
                
            Returns:
                str: 处理结果
            """
            return "documented"
        
        analyzer = FuncAnalyser(documented_func)
        descriptor = analyzer.func_descriptor
        
        assert descriptor.func is documented_func
        assert descriptor.func.__doc__ is not None
        assert "这是一个有文档的函数" in descriptor.func.__doc__
    
    def test_analyzer_function_no_parameters(self):
        """测试分析无参数函数"""
        def no_param_func():
            """无参数函数"""
            return "no params"
        
        analyzer = FuncAnalyser(no_param_func)
        descriptor = analyzer.func_descriptor
        
        assert len(descriptor.param_list) == 0
        assert len(descriptor.param_names) == 0
        assert len(descriptor.param_annotations) == 0



from .event_data import BaseEventData, MessageEventData
from .message import BaseMessageEvent, AnonymousMessage, GroupMessageEvent, PrivateMessageEvent
from .meta import MetaEvent, Status
from .notice import NoticeEvent
from .request import RequestEvent
from .sender import BaseSender, PrivateSender, GroupSender
from .message_segment import (
    MessageSegment,
    MessageArray,
    Text,
    PlainText,
    Face,
    Image,
    File,
    Record,
    Video,
    At,
    AtAll,
    Rps,
    Dice,
    Shake,
    Poke,
    Anonymous,
    Share,
    Contact,
    Location,
    Music,
    Reply,
    Node,
    Forward,
    XML,
    Json,
)

__all__ = [
    # From event_data.py
    'BaseEventData',
    'MessageEventData',
    
    # From message.py
    'BaseMessageEvent',
    'AnonymousMessage',
    'GroupMessageEvent',
    'PrivateMessageEvent',
    
    # From meta.py
    'MetaEvent',
    'Status',
    
    # From notice.py
    'NoticeEvent',
    'NoticeFile',
    
    # From request.py
    'RequestEvent',
    
    # From sender.py
    'BaseSender',
    'PrivateSender',
    'GroupSender',
    
    # From message_segment
    'MessageSegment',
    'MessageArray',
    'Text',
    'PlainText',
    'Face',
    'Image',
    'File',
    'Record',
    'Video',
    'At',
    'AtAll',
    'Rps',
    'Dice',
    'Shake',
    'Poke',
    'Anonymous',
    'Share',
    'Contact',
    'Location',
    'Music',
    'Reply',
    'Node',
    'Forward',
    'XML',
    'Json',
]


"""sys management commands for NcatBot CLI."""

import subprocess
import sys
import time

from ncatbot.cli.commands.registry import registry
from ncatbot.cli.utils import PYPI_SOURCE, CLIExit
from ncatbot.cli.utils.colors import error, header, info, success
from ncatbot.core import BotClient, install_napcat
from ncatbot.utils.logger import get_log

LOG = get_log("CLI")


@registry.register(
    "start",
    "启动 NcatBot",
    "start [-d|-D|--debug]",
    aliases=["s", "run"],
    category="sys",
)
def start(*args: str) -> None:
    """Start the NcatBot client."""
    print(header("正在启动 NcatBot..."))
    print(info("按下 Ctrl + C 可以正常退出程序"))

    try:
        # 创建客户端
        client = BotClient()  # 密码暂时为空
        # 启动客户端（带调试选项）
        client.run()

    except Exception as e:
        LOG.error(f"启动失败: {e}")
        print(error(f"启动失败: {e}"))


@registry.register(
    "update",
    "更新 NcatBot 和 NapCat",
    "update",
    aliases=["u", "upgrade"],
    category="sys",
    show_in_help=False,
)
def update() -> None:
    """Update NcatBot and NapCat."""
    print(header("正在更新 NapCat 版本"))
    install_napcat()

    print(header("正在更新 Ncatbot 版本, 更新后请重新运行 NcatBotCLI 或者 main.exe"))
    time.sleep(1)

    subprocess.Popen(
        [
            sys.executable,
            "-m",
            "pip",
            "install",
            "--upgrade",
            "ncatbot",
            "-i",
            PYPI_SOURCE,
        ],
        shell=True,
        start_new_session=True,
    )

    print(info("Ncatbot 正在更新..."))
    time.sleep(10)
    print(success("更新成功, 请重新运行 NcatBotCLI 或者 main.exe"))
    exit(0)


@registry.register(
    "exit",
    "退出 CLI 工具",
    "exit",
    aliases=["quit", "q"],
    category="sys",
    show_in_help=False,
)
def exit_cli() -> None:
    """Exit the CLI tool."""
    print("\n" + info("正在退出 Ncatbot CLI. 再见!"))
    raise CLIExit()



"""Command registry system for NcatBot CLI."""

from typing import Any, Callable, Dict, List, Optional, Tuple

from ncatbot.cli.utils.colors import (
    Colors,
)
from ncatbot.cli.utils.colors import aliases as alias_color
from ncatbot.cli.utils.colors import category as cat_color
from ncatbot.cli.utils.colors import command as cmd_color
from ncatbot.cli.utils.colors import description as desc_color
from ncatbot.cli.utils.colors import (
    get_category_color,
)
from ncatbot.cli.utils.colors import header as header_color


class Command:
    """Command class to represent a CLI command"""

    def __init__(
        self,
        name: str,
        func: Callable,
        description: str,
        usage: str,
        help_text: Optional[str] = None,
        aliases: Optional[List[str]] = None,
        category: Optional[str] = None,
        show_in_help: bool = True,
    ):
        self.name = name
        self.func = func
        self.description = description
        self.usage = usage
        self.help_text = help_text or description
        self.aliases = aliases or []
        self.category = category or "General"
        self.show_in_help = show_in_help


class CommandRegistry:
    """Registry for CLI commands"""

    def __init__(self):
        self.commands: Dict[str, Command] = {}
        self.aliases: Dict[str, str] = {}
        self.categories: Dict[str, List[str]] = {}

    def register(
        self,
        name: str,
        description: str,
        usage: str,
        help_text: Optional[str] = None,
        aliases: Optional[List[str]] = None,
        category: Optional[str] = None,
        requires_qq: bool = False,
        show_in_help: bool = True,
    ):
        """Decorator to register a command"""

        def decorator(func: Callable) -> Callable:
            cmd = Command(
                name=name,
                func=func,
                description=description,
                usage=usage,
                help_text=help_text,
                aliases=aliases,
                category=category,
                show_in_help=show_in_help,
            )
            # Store requires_qq as an attribute of the command
            self.commands[name] = cmd

            # Register aliases
            if aliases:
                for aliases in aliases:
                    self.aliases[aliases] = name

            # Register category
            if category:
                if category not in self.categories:
                    self.categories[category] = []
                self.categories[category].append(name)

            return func

        return decorator

    def execute(self, command_name: str, *args, **kwargs) -> Any:
        """Execute a command by name"""
        # Check if the command is an aliases
        if command_name in self.aliases:
            command_name = self.aliases[command_name]

        if command_name not in self.commands:
            print(f"不支持的命令: {cmd_color(command_name)}")
            return None

        cmd = self.commands[command_name]
        return cmd.func(*args, **kwargs)

    def get_help(
        self, category: Optional[str] = None, only_important: bool = False
    ) -> str:
        """Generate help text for all commands or a specific category

        Args:
            category: Optional category to filter commands by
            only_important: If True, only show commands with show_in_help=True
        """
        if category and category not in self.categories:
            return f"未知的分类: {cat_color(category)}"

        help_lines = []
        if category:
            cat_title = cat_color(category)
            help_lines.append(f"{cat_title} 分类的命令:")
            commands = [
                (name, self.commands[name])
                for name in self.categories[category]
                if not only_important or self.commands[name].show_in_help
            ]
        else:
            all_commands = sorted(self.commands.items())
            if only_important:
                commands = [
                    (name, cmd) for name, cmd in all_commands if cmd.show_in_help
                ]
            else:
                commands = all_commands

            if commands:
                help_lines.append(header_color("支持的命令:"))

        for i, (name, cmd) in enumerate(commands, 1):
            # Include aliases in the help text if they exist
            alias_text = ""
            if cmd.aliases:
                aliases = [alias_color(a) for a in cmd.aliases]
                alias_text = f" (别名: {', '.join(aliases)})"

            # Get category color for this command
            cat_color_code = get_category_color(cmd.category)
            # Format the command with its category's color
            colored_usage = f"{Colors.BOLD}{cat_color_code}{cmd.usage}{Colors.RESET}"

            help_lines.append(
                f"{i}. '{colored_usage}' - {desc_color(cmd.description)}{alias_text}"
            )

        return "\n".join(help_lines)

    def get_categories(self) -> List[str]:
        """Get list of all command categories"""
        return sorted(self.categories.keys())

    def get_commands_by_category(
        self, category: str, only_important: bool = False
    ) -> List[Tuple[str, Command]]:
        """Get all commands in a specific category

        Args:
            category: The category to get commands for
            only_important: If True, only return commands with show_in_help=True
        """
        if category not in self.categories:
            return []
        commands = [(name, self.commands[name]) for name in self.categories[category]]
        if only_important:
            return [(name, cmd) for name, cmd in commands if cmd.show_in_help]
        return commands


# Create a global command registry
registry = CommandRegistry()



"""过滤器系统集成测试

测试过滤器系统各组件的集成功能，包括过滤器链执行、与命令的集成、装饰器使用等。
"""

import pytest
import asyncio
from unittest.mock import Mock, patch

from ncatbot.core.event import BaseMessageEvent, GroupMessageEvent, PrivateMessageEvent
from ncatbot.plugin_system.builtin_plugin.unified_registry.filter_system import (
    filter_registry, GroupFilter, PrivateFilter, AdminFilter, CustomFilter,
    group_only, private_only, admin_only
)


class TestFilterChainExecution:
    """过滤器链执行测试"""
    
    @pytest.mark.asyncio
    async def test_simple_filter_chain(self, mock_group_message, clean_registries):
        """测试简单过滤器链执行"""
        execution_order = []
        
        # 创建记录执行顺序的过滤器
        def filter1(event: BaseMessageEvent):
            execution_order.append("filter1")
            return True
        
        def filter2(event: BaseMessageEvent):
            execution_order.append("filter2")
            return True
        
        # 注册过滤器
        filter_registry.register(filter1, "filter1")
        filter_registry.register(filter2, "filter2")
        
        # 创建带过滤器的函数
        def test_function(event: BaseMessageEvent):
            execution_order.append("function")
            return "executed"
        
        # 手动添加过滤器
        filter_registry.add_filter_to_function(test_function, "filter1", "filter2")
        
        # 验证过滤器已添加
        assert hasattr(test_function, '__filters__')
        assert len(test_function.__filters__) == 2
    
    @pytest.mark.asyncio
    async def test_filter_chain_early_termination(self, mock_private_message, clean_registries):
        """测试过滤器链早期终止"""
        execution_order = []
        
        # 第一个过滤器通过
        def filter1(event: BaseMessageEvent):
            execution_order.append("filter1")
            return True
        
        # 第二个过滤器失败
        def filter2(event: BaseMessageEvent):
            execution_order.append("filter2")
            return False
        
        # 第三个过滤器不应该被执行
        def filter3(event: BaseMessageEvent):
            execution_order.append("filter3")
            return True
        
        # 注册过滤器
        filter_registry.register(filter1, "filter1")
        filter_registry.register(filter2, "filter2") 
        filter_registry.register(filter3, "filter3")
        
        # 创建带过滤器的函数
        def test_function(event: BaseMessageEvent):
            execution_order.append("function")
            return "executed"
        
        filter_registry.add_filter_to_function(test_function, "filter1", "filter2", "filter3")
        
        # 模拟验证器行为
        from ncatbot.plugin_system.builtin_plugin.unified_registry.filter_system.validator import FilterValidator
        validator = FilterValidator()
        
        result = validator.validate_filters(test_function, mock_private_message)
        
        # 应该在filter2失败时停止
        assert result is False
    
    @pytest.mark.asyncio
    async def test_mixed_filter_types_chain(self, mock_group_message, mock_status_manager, clean_registries):
        """测试混合过滤器类型链"""
        # 内置过滤器
        group_filter = GroupFilter()
        admin_filter = AdminFilter()
        
        # 自定义过滤器
        def time_filter(event: BaseMessageEvent):
            return True  # 简化为总是通过
        
        custom_filter = CustomFilter(time_filter, "time_check")
        
        # 模拟管理员权限
        mock_status_manager.user_has_role.return_value = True
        
        # 注册自定义过滤器
        filter_registry.register_filter("time_check", custom_filter)
        filter_registry.register_filter("group_check", group_filter)
        filter_registry.register_filter("admin_check", admin_filter)
        
        def test_function(event: BaseMessageEvent):
            return "all filters passed"
        
        # 添加混合过滤器
        filter_registry.add_filter_to_function(
            test_function, 
            "group_check", 
            "admin_check", 
            "time_check"
        )
        
        # 验证所有过滤器
        from ncatbot.plugin_system.builtin_plugin.unified_registry.filter_system.validator import FilterValidator
        validator = FilterValidator()
        
        result = validator.validate_filters(test_function, mock_group_message)
        assert result is True


class TestFilterWithCommand:
    """过滤器与命令集成测试"""
    
    @pytest.mark.asyncio
    async def test_filter_command_integration(self, unified_plugin, mock_group_message, clean_registries):
        """测试过滤器与命令的集成"""
        from ncatbot.plugin_system import command_registry
        
        # 注册带过滤器的命令
        @command_registry.command("admin_cmd")
        def admin_command(event: BaseMessageEvent):
            return "Admin command executed"
        admin_command.__is_command__ = True
        
        # 添加管理员过滤器
        admin_filter = AdminFilter()
        filter_registry.add_filter_to_function(admin_command, admin_filter)
        
        # 模拟执行
        with patch.object(unified_plugin, '_filter_validator') as mock_validator:
            mock_validator.validate_filters.return_value = True
            
            result = await unified_plugin._execute_function(admin_command, mock_group_message)
            
            # 应该调用过滤器验证
            mock_validator.validate_filters.assert_called_once_with(admin_command, mock_group_message)
    
    @pytest.mark.asyncio
    async def test_filter_command_rejection(self, unified_plugin, mock_private_message, clean_registries):
        """测试过滤器拒绝命令执行"""
        from ncatbot.plugin_system import command_registry
        
        # 注册带群聊过滤器的命令
        @command_registry.command("group_only_cmd")
        def group_only_command(event: BaseMessageEvent):
            return "Should not execute"
        group_only_command.__is_command__ = True
        
        # 添加群聊过滤器
        group_filter = GroupFilter()
        filter_registry.add_filter_to_function(group_only_command, group_filter)
        
        # 模拟执行（私聊消息应该被拒绝）
        with patch.object(unified_plugin, '_filter_validator') as mock_validator:
            mock_validator.validate_filters.return_value = False
            
            result = await unified_plugin._execute_function(group_only_command, mock_private_message)
            
            # 应该被过滤器拒绝
            assert result is False


class TestFilterDecorators:
    """过滤器装饰器测试"""
    
    def test_group_only_decorator(self, mock_group_message, mock_private_message, clean_registries):
        """测试群聊专用装饰器"""
        @group_only
        def group_function(event: BaseMessageEvent):
            return "group only"
        
        # 检查过滤器是否添加
        assert hasattr(group_function, '__filters__')
        assert len(group_function.__filters__) > 0
        assert any(isinstance(f, GroupFilter) for f in group_function.__filters__)
    
    def test_private_only_decorator(self, mock_private_message, mock_group_message, clean_registries):
        """测试私聊专用装饰器"""
        @private_only
        def private_function(event: BaseMessageEvent):
            return "private only"
        
        # 检查过滤器是否添加
        assert hasattr(private_function, '__filters__')
        assert len(private_function.__filters__) > 0
        assert any(isinstance(f, PrivateFilter) for f in private_function.__filters__)
    
    def test_admin_only_decorator(self, mock_private_message, mock_status_manager, clean_registries):
        """测试管理员专用装饰器"""
        @admin_only
        def admin_function(event: BaseMessageEvent):
            return "admin only"
        
        # 检查过滤器是否添加
        assert hasattr(admin_function, '__filters__')
        assert len(admin_function.__filters__) > 0
        assert any(isinstance(f, AdminFilter) for f in admin_function.__filters__)
    
    def test_chained_decorators(self, mock_group_message, mock_status_manager, clean_registries):
        """测试链式装饰器"""
        @admin_only
        @group_only
        def admin_group_function(event: BaseMessageEvent):
            return "admin group only"
        
        # 应该有两个过滤器
        assert hasattr(admin_group_function, '__filters__')
        assert len(admin_group_function.__filters__) >= 2
        
        # 应该包含两种类型的过滤器
        filter_types = [type(f) for f in admin_group_function.__filters__]
        assert GroupFilter in filter_types or any(isinstance(f, GroupFilter) for f in admin_group_function.__filters__)
        assert AdminFilter in filter_types or any(isinstance(f, AdminFilter) for f in admin_group_function.__filters__)


class TestFilterRegistryGlobalState:
    """过滤器注册器全局状态测试"""
    
    def test_global_filter_registry(self, clean_registries):
        """测试全局过滤器注册器"""
        # 使用全局注册器
        @filter_registry.register("global_test")
        def global_filter(event: BaseMessageEvent):
            return True
        
        # 应该在全局注册器中找到
        entry = filter_registry.get_filter("global_test")
        assert entry is not None
        assert entry.name == "global_test"
    
    def test_registry_state_persistence(self, clean_registries):
        """测试注册器状态持久性"""
        # 注册一个过滤器
        test_filter = GroupFilter()
        filter_registry.register_filter("persistent_test", test_filter)
        
        # 在不同的函数中应该能找到
        entry1 = filter_registry.get_filter("persistent_test")
        entry2 = filter_registry.get_filter("persistent_test")
        
        assert entry1 is entry2
        assert entry1.filter_instance is test_filter
    
    def test_registry_cleanup(self, clean_registries):
        """测试注册器清理"""
        # 注册一些测试数据
        filter_registry.register_filter("cleanup_test1", GroupFilter())
        filter_registry.register_filter("cleanup_test2", PrivateFilter())
        
        def test_func(event: BaseMessageEvent):
            return True
        filter_registry.register(test_func, "func_test")
        
        # 验证数据存在
        assert "cleanup_test1" in filter_registry._filters
        assert "cleanup_test2" in filter_registry._filters
        assert "func_test" in filter_registry._filters
        assert test_func in filter_registry._function_filters
        
        # clean_registries fixture 应该在测试后清理


class TestComplexFilterScenarios:
    """复杂过滤器场景测试"""
    
    @pytest.mark.asyncio
    async def test_conditional_filter_chain(self, mock_group_message, clean_registries):
        """测试条件过滤器链"""
        execution_log = []
        
        # 条件过滤器：只在群聊中检查特定条件
        def conditional_filter(event: BaseMessageEvent):
            execution_log.append("conditional")
            if event.is_group_msg():
                return len(event.user_id) > 3
            return True
        
        # 基础群聊过滤器
        group_filter = GroupFilter()
        
        # 注册过滤器
        filter_registry.register_filter("group", group_filter)
        filter_registry.register(conditional_filter, "conditional")
        
        def test_function(event: BaseMessageEvent):
            execution_log.append("function")
            return "executed"
        
        filter_registry.add_filter_to_function(test_function, "group", "conditional")
        
        # 测试用户ID长度足够的情况
        mock_group_message.user_id = "123456"
        
        from ncatbot.plugin_system.builtin_plugin.unified_registry.filter_system.validator import FilterValidator
        validator = FilterValidator()
        result = validator.validate_filters(test_function, mock_group_message)
        assert result is True
        
        # 测试用户ID长度不够的情况
        execution_log.clear()
        mock_group_message.user_id = "12"
        result = validator.validate_filters(test_function, mock_group_message)
        assert result is False
    
    @pytest.mark.asyncio
    async def test_dynamic_filter_modification(self, mock_private_message, clean_registries):
        """测试动态过滤器修改"""
        # 创建可修改状态的过滤器
        class DynamicFilter:
            def __init__(self):
                self.enabled = True
            
            def check(self, event):
                return self.enabled
        
        dynamic_filter = DynamicFilter()
        
        # 包装为CustomFilter
        def dynamic_check(event: BaseMessageEvent):
            return dynamic_filter.check(event)
        
        custom_filter = CustomFilter(dynamic_check, "dynamic")
        
        def test_function(event: BaseMessageEvent):
            return "executed"
        
        filter_registry.register_filter("dynamic", custom_filter)
        filter_registry.add_filter_to_function(test_function, "dynamic")
        
        from ncatbot.plugin_system.builtin_plugin.unified_registry.filter_system.validator import FilterValidator
        validator = FilterValidator()
        
        # 启用状态应该通过
        result = validator.validate_filters(test_function, mock_private_message)
        assert result is True
        
        # 禁用状态应该失败
        dynamic_filter.enabled = False
        result = validator.validate_filters(test_function, mock_private_message)
        assert result is False
    
    @pytest.mark.asyncio
    async def test_filter_error_recovery(self, mock_private_message, clean_registries):
        """测试过滤器错误恢复"""
        error_count = 0
        
        def error_prone_filter(event: BaseMessageEvent):
            nonlocal error_count
            error_count += 1
            if error_count <= 2:
                raise ValueError(f"Error {error_count}")
            return True
        
        filter_registry.register(error_prone_filter, "error_prone")
        
        def test_function(event: BaseMessageEvent):
            return "executed"
        
        filter_registry.add_filter_to_function(test_function, "error_prone")
        
        from ncatbot.plugin_system.builtin_plugin.unified_registry.filter_system.validator import FilterValidator
        validator = FilterValidator()
        
        # 前两次应该抛出异常
        
        assert validator.validate_filters(test_function, mock_private_message) is False
        
        assert validator.validate_filters(test_function, mock_private_message) is False
        
        # 第三次应该成功
        result = validator.validate_filters(test_function, mock_private_message)
        assert result is True



"""Color utilities for NcatBot CLI."""

import os
import sys
from typing import Dict


# ANSI color codes
class Colors:
    """ANSI color codes for terminal output."""

    # Text colors
    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"
    GRAY = "\033[90m"

    # Background colors
    BG_BLACK = "\033[40m"
    BG_RED = "\033[41m"
    BG_GREEN = "\033[42m"
    BG_YELLOW = "\033[43m"
    BG_BLUE = "\033[44m"
    BG_MAGENTA = "\033[45m"
    BG_CYAN = "\033[46m"
    BG_WHITE = "\033[47m"

    # Text styles
    BOLD = "\033[1m"
    ITALIC = "\033[3m"
    UNDERLINE = "\033[4m"

    # Reset
    RESET = "\033[0m"


# Semantic color scheme
class ColorScheme:
    """Semantic color scheme for NcatBot CLI."""

    HEADER = Colors.BOLD + Colors.CYAN
    TITLE = Colors.BOLD + Colors.BLUE
    COMMAND = Colors.BOLD + Colors.GREEN
    CATEGORY = Colors.BOLD + Colors.YELLOW
    DESCRIPTION = Colors.WHITE
    USAGE = Colors.CYAN
    aliases = Colors.MAGENTA
    ERROR = Colors.RED
    SUCCESS = Colors.GREEN
    WARNING = Colors.YELLOW
    INFO = Colors.BLUE
    RESET = Colors.RESET


# Check if colors should be enabled (can be disabled for non-ANSI terminals)
def is_color_supported() -> bool:
    """Check if the terminal supports ANSI color codes."""
    # Check if FORCE_COLOR is set to disable colors
    if os.environ.get("FORCE_COLOR", "").lower() in ("0", "false", "no"):
        return False

    # Check if NO_COLOR is set (standard for disabling colors)
    if os.environ.get("NO_COLOR") is not None:
        return False

    # Check if it's Windows without colorama
    if sys.platform == "win32" and not (
        "ANSICON" in os.environ or "WT_SESSION" in os.environ
    ):
        try:
            import colorama

            colorama.init()
            return True
        except ImportError:
            return False

    # Most Unix terminals support colors
    return True


# Get color function based on terminal support
_color_enabled = is_color_supported()


def colorize(text: str, color: str, always: bool = False) -> str:
    """Colorize text string with ANSI color codes if colors are enabled.

    Args:
        text: Text to colorize
        color: ANSI color code
        always: If True, always apply color even if colors are disabled

    Returns:
        Colorized text string
    """
    if _color_enabled or always:
        return f"{color}{text}{Colors.RESET}"
    return text


# Generate convenient color functions
def command(text: str) -> str:
    """Format text as a command."""
    return colorize(text, ColorScheme.COMMAND)


def category(text: str) -> str:
    """Format text as a category."""
    return colorize(text, ColorScheme.CATEGORY)


def description(text: str) -> str:
    """Format text as a description."""
    return colorize(text, ColorScheme.DESCRIPTION)


def usage(text: str) -> str:
    """Format text as a usage example."""
    return colorize(text, ColorScheme.USAGE)


def aliases(text: str) -> str:
    """Format text as an aliases."""
    return colorize(text, ColorScheme.aliases)


def error(text: str) -> str:
    """Format text as an error message."""
    return colorize(text, ColorScheme.ERROR)


def success(text: str) -> str:
    """Format text as a success message."""
    return colorize(text, ColorScheme.SUCCESS)


def warning(text: str) -> str:
    """Format text as a warning message."""
    return colorize(text, ColorScheme.WARNING)


def info(text: str) -> str:
    """Format text as an info message."""
    return colorize(text, ColorScheme.INFO)


def header(text: str) -> str:
    """Format text as a header."""
    return colorize(text, ColorScheme.HEADER)


def title(text: str) -> str:
    """Format text as a title."""
    return colorize(text, ColorScheme.TITLE)


# Category-specific colors
CATEGORY_COLORS: Dict[str, str] = {
    "sys": Colors.YELLOW,
    "info": Colors.CYAN,
    "plg": Colors.GREEN,
}


def get_category_color(category_name: str) -> str:
    """Get color for a specific category.

    Args:
        category_name: Name of the category

    Returns:
        ANSI color code for the category
    """
    return CATEGORY_COLORS.get(category_name.lower(), Colors.WHITE)



"""完整工作流测试

测试unified_registry模块的完整工作流程，从消息接收到命令执行的端到端测试。
"""

import pytest
import asyncio
from unittest.mock import Mock, patch, AsyncMock

from ncatbot.plugin_system.builtin_plugin.unified_registry import UnifiedRegistryPlugin
from ncatbot.plugin_system.builtin_plugin.unified_registry.filter_system import (
    filter_registry, GroupFilter, PrivateFilter, AdminFilter
)
from ncatbot.plugin_system import command_registry
from ncatbot.core.event import BaseMessageEvent
from ncatbot.utils.testing import EventFactory
from ncatbot.core.event.message_segment import MessageArray, Text, At


class TestFullWorkflow:
    """完整工作流测试"""
    
    @pytest.mark.asyncio
    async def test_simple_command_full_workflow(self, unified_plugin, clean_registries):
        """测试简单命令的完整工作流"""
        execution_log = []
        
        # 1. 注册命令
        @command_registry.command("hello", description="问候命令")
        def hello_command(event: BaseMessageEvent):
            execution_log.append("hello_executed")
            return "Hello World!"
        hello_command.__is_command__ = True
        
        # 2. 创建消息事件
        mock_event = EventFactory.create_private_message(message="/hello", user_id="test_user")
        
        # 3. 处理消息事件
        await unified_plugin.handle_message_event(mock_event)

        # 4. 验证执行
        assert "hello_executed" in execution_log
    
    @pytest.mark.asyncio
    async def test_command_with_parameters_workflow(self, unified_plugin, clean_registries):
        """测试带参数命令的完整工作流"""
        execution_results = []
        
        # 1. 注册带参数的命令
        @command_registry.command("greet", description="个性化问候")
        def greet_command(event: BaseMessageEvent, name: str, age: int = 18):
            result = f"Hello {name}, you are {age} years old!"
            execution_results.append(result)
            return result
        greet_command.__is_command__ = True
        
        # 2. 创建带参数的消息事件
        mock_event = EventFactory.create_private_message(message="/greet Alice 25", user_id="test_user")
        
        # 3. 模拟参数绑定和执行
        await unified_plugin.handle_message_event(mock_event)
            
        assert "Hello Alice" in "".join(execution_results)
        assert "25 years old" in "".join(execution_results)
    
    
    @pytest.mark.asyncio
    async def test_filtered_command_workflow(self, unified_plugin, clean_registries, mock_status_manager):
        """测试带过滤器命令的完整工作流"""
        execution_log = []
        
        # 1. 注册带过滤器的命令
        @command_registry.command("admin_only", description="管理员专用命令")
        def admin_command(event: BaseMessageEvent):
            execution_log.append("admin_command_executed")
            return "Admin operation completed"
        admin_command.__is_command__ = True
        
        # 2. 添加管理员过滤器
        admin_filter = AdminFilter()
        filter_registry.add_filter_to_function(admin_command, admin_filter)
        
        # 3. 测试非管理员用户
        mock_status_manager.user_has_role.return_value = False
        
        mock_event = EventFactory.create_private_message(message="/admin_only", user_id="normal_user")
        
        await unified_plugin.handle_message_event(mock_event)
        
        # 非管理员不应该执行命令
        assert "admin_command_executed" not in execution_log
        
        # 4. 测试管理员用户
        execution_log.clear()
        mock_status_manager.user_has_role.return_value = True
        
        mock_event = EventFactory.create_private_message(message="/admin_only", user_id="admin_user")
        
        with patch.object(unified_plugin, '_execute_function') as mock_execute:
            mock_execute.return_value = "Admin operation completed"
            
            await unified_plugin.handle_message_event(mock_event)
            
            # 管理员应该能执行命令
            mock_execute.assert_called()
    
    @pytest.mark.asyncio
    async def test_hierarchical_command_workflow(self, unified_plugin, clean_registries):
        """测试层级命令的完整工作流"""
        execution_log = []
        
        # 1. 注册层级命令
        admin_group = command_registry.group("admin", description="管理员命令组")
        user_group = admin_group.group("user", description="用户管理")
        
        @user_group.command("ban", description="封禁用户")
        def ban_user_command(event: BaseMessageEvent, target_user: str):
            execution_log.append(f"banned_{target_user}")
            return f"User {target_user} has been banned"
        ban_user_command.__is_command__ = True
        
        # 2. 创建层级命令消息
        mock_event = EventFactory.create_private_message(message="/admin user ban target123", user_id="admin_user")
        
        # 3. 模拟执行
        with patch.object(unified_plugin, '_execute_function') as mock_execute:
            mock_execute.return_value = "User target123 has been banned"
            
            await unified_plugin.handle_message_event(mock_event)
            
            # 验证层级命令被正确解析和执行
            mock_execute.assert_called()
    
    @pytest.mark.asyncio
    async def test_complex_message_workflow(self, unified_plugin, clean_registries):
        """测试复杂消息的完整工作流"""
        execution_log = []
        
        # 1. 注册处理复杂消息的命令
        @command_registry.command("send", description="发送消息给用户")
        def send_command(event: BaseMessageEvent, target: str, message_text: str):
            execution_log.append(f"sent_to_{target}_{message_text}")
            return f"Message sent to {target}: {message_text}"
        send_command.__is_command__ = True
        
        # 2. 创建包含@和文本的复杂消息
        segments = [
            Text("/send "),
            At("123456"),  # @用户
            Text(" hello world")
        ]
        message_array = MessageArray(*segments)
        mock_event = EventFactory.create_private_message(message=message_array, user_id="sender")
        
        # 3. 模拟执行
        with patch.object(unified_plugin, '_execute_function') as mock_execute:
            mock_execute.return_value = "Message sent"
            
            await unified_plugin.handle_message_event(mock_event)
            
            # 验证复杂消息被正确处理
            mock_execute.assert_called()
    
    @pytest.mark.asyncio
    async def test_alias_command_workflow(self, unified_plugin, clean_registries):
        """测试别名命令的完整工作流"""
        execution_log = []
        
        # 1. 注册带别名的命令
        @command_registry.command("status", aliases=["st", "stat"], description="查看状态")
        def status_command(event: BaseMessageEvent):
            execution_log.append("status_checked")
            return "System status: OK"
        status_command.__is_command__ = True
        
        # 2. 测试主命令名
        mock_event_1 = EventFactory.create_private_message(message="/status", user_id="test_user")
        await unified_plugin.handle_message_event(mock_event_1)
        assert "status_checked" in execution_log
        execution_log.clear()
        mock_event_2 = EventFactory.create_private_message(message="/st", user_id="test_user")
        await unified_plugin.handle_message_event(mock_event_2)
        assert "status_checked" in execution_log
        assert "status_checked" in execution_log
        execution_log.clear()
        mock_event_3 = EventFactory.create_private_message(message="/stat", user_id="test_user")
        await unified_plugin.handle_message_event(mock_event_3)
        assert "status_checked" in execution_log
        assert "status_checked" in execution_log
        
    
    @pytest.mark.asyncio
    async def test_filter_only_workflow(self, unified_plugin, clean_registries):
        """测试纯过滤器（非命令）的完整工作流"""
        execution_log = []
        
        # 1. 注册纯过滤器函数
        @filter_registry.register("log_filter")
        def log_all_messages(event: BaseMessageEvent):
            execution_log.append(f"logged_{event.user_id}")
            return True
        
        # 2. 创建普通消息（非命令）
        mock_event = EventFactory.create_private_message(message="hello everyone", user_id="test_user")
        
        # 3. 模拟执行
        with patch.object(unified_plugin, '_execute_function') as mock_execute:
            mock_execute.return_value = True
            
            await unified_plugin.handle_message_event(mock_event)
            
            # 验证过滤器被执行
            mock_execute.assert_called()
    
    @pytest.mark.asyncio
    async def test_no_match_workflow(self, unified_plugin, clean_registries):
        """测试无匹配的完整工作流"""
        # 1. 不注册任何命令
        
        # 2. 创建命令格式的消息
        mock_event = EventFactory.create_private_message(message="/nonexistent", user_id="test_user")
        
        # 3. 处理消息
        result = await unified_plugin.handle_message_event(mock_event)
        
        # 4. 应该正常处理，不抛出异常
        # 结果可能是None或False，表示无匹配
        assert result is None or result is False
    
    @pytest.mark.asyncio
    async def test_error_handling_workflow(self, unified_plugin, clean_registries):
        """测试错误处理的完整工作流"""
        # 1. 注册会抛异常的命令
        @command_registry.command("error_cmd", description="会出错的命令")
        def error_command(event: BaseMessageEvent):
            raise ValueError("Command error")
        error_command.__is_command__ = True
        
        # 2. 创建消息事件
        mock_event = EventFactory.create_private_message(message="/error_cmd", user_id="test_user")
        
        # 3. 模拟执行
        # 应该能处理错误，不崩溃
        result = await unified_plugin.handle_message_event(mock_event)
        
        # 错误应该被捕获
        assert result is None or result is False


class TestAsyncWorkflow:
    """异步工作流测试"""
    
    @pytest.mark.asyncio
    async def test_async_command_workflow(self, unified_plugin, clean_registries):
        """测试异步命令的完整工作流"""
        execution_log = []
        
        # 1. 注册异步命令
        @command_registry.command("async_task", description="异步任务")
        async def async_command(event: BaseMessageEvent, delay: float = 0.01):
            await asyncio.sleep(delay)
            execution_log.append("async_task_completed")
            return "Async task completed"
        async_command.__is_command__ = True
        
        # 2. 创建消息事件
        mock_event = EventFactory.create_private_message(message="/async_task 0.01", user_id="test_user")
        
        # 3. 模拟执行
        with patch.object(unified_plugin, '_execute_function') as mock_execute:
            # 模拟异步执行
            async def async_execute(*args, **kwargs):
                return await async_command(mock_event, 0.01)
            
            mock_execute.side_effect = async_execute
            
            await unified_plugin.handle_message_event(mock_event)
            
            # 验证异步命令被执行
            mock_execute.assert_called()
            assert "async_task_completed" in execution_log
    
    @pytest.mark.asyncio
    async def test_concurrent_message_handling(self, unified_plugin, clean_registries):
        """测试并发消息处理"""
        execution_count = []
        
        # 1. 注册命令
        @command_registry.command("concurrent", description="并发测试")
        async def concurrent_command(event: BaseMessageEvent, task_id: str):
            await asyncio.sleep(0.01)  # 模拟异步操作
            execution_count.append(task_id)
            return f"Task {task_id} completed"
        concurrent_command.__is_command__ = True
        
        # 2. 创建多个消息事件
        events = []
        for i in range(5):
            mock_event = EventFactory.create_private_message(message=f"/concurrent task{i}", user_id=f"user{i}")
            events.append(mock_event)
        
        # 3. 并发处理
        with patch.object(unified_plugin, '_execute_function') as mock_execute:
            # 模拟并发执行
            async def concurrent_execute(func, event, *args, **kwargs):
                return await func(event, *args, **kwargs)
            
            mock_execute.side_effect = concurrent_execute
            
            # 并发处理所有消息
            tasks = [unified_plugin.handle_message_event(event) for event in events]
            await asyncio.gather(*tasks)
            
            # 验证所有任务都被执行
            assert len(execution_count) == 5
            assert all(f"task{i}" in execution_count for i in range(5))


class TestWorkflowWithRealComponents:
    """使用真实组件的工作流测试"""
    
    @pytest.mark.asyncio
    async def test_end_to_end_real_workflow(self, unified_plugin, clean_registries):
        """端到端真实工作流测试"""
        execution_log = []
        
        # 1. 注册真实命令和过滤器
        @command_registry.command("deploy", aliases=["d"], description="部署应用")
        def deploy_command(event: BaseMessageEvent, app_name: str, env: str = "dev"):
            execution_log.append(f"deployed_{app_name}_to_{env}")
            return f"Successfully deployed {app_name} to {env}"
        deploy_command.__is_command__ = True
        
        # 添加群聊过滤器
        group_filter = GroupFilter()
        filter_registry.add_filter_to_function(deploy_command, group_filter)
        
        # 2. 创建群聊消息事件
        mock_event = EventFactory.create_group_message(
            message="/deploy myapp prod",
            user_id="deployer",
            group_id="dev_group"
        )
        
        # 3. 让unified_plugin处理真实的初始化和执行
        # 注意：这里不模拟内部方法，让它们真实执行
        result = await unified_plugin.handle_message_event(mock_event)
        
        # 4. 验证处理完成（具体结果取决于实现）
        # 由于是真实工作流，结果可能因实现细节而异
        assert result is None or isinstance(result, bool)
    
    @pytest.mark.asyncio
    async def test_plugin_initialization_workflow(self, test_client):
        """测试插件初始化工作流"""
        # 1. 注册插件
        test_client.register_plugin(UnifiedRegistryPlugin)
        
        # 2. 获取插件实例
        plugins = test_client.get_registered_plugins()
        unified_plugin = next((p for p in plugins if isinstance(p, UnifiedRegistryPlugin)), None)
        assert unified_plugin is not None
        
        # 3. 验证插件状态
        assert unified_plugin.name == "UnifiedRegistryPlugin"
        assert unified_plugin._preprocessor is not None
        assert unified_plugin._resolver is not None
        assert unified_plugin._binder is not None
        
        # 4. 清理
        test_client.unregister_plugin(unified_plugin)
    
    @pytest.mark.asyncio
    async def test_multi_plugin_interaction_workflow(self, test_client, clean_registries):
        """测试多插件交互工作流"""
        execution_log = []
        
        # 1. 创建辅助插件
        from ncatbot.plugin_system import BasePlugin
        
        class HelperPlugin(BasePlugin):
            name = "HelperPlugin"
            version = "1.0.0"
            
            async def on_load(self):
                self.data = {"helper": "ready"}
            
            def get_helper_data(self):
                return self.data
        
        # 2. 注册插件
        test_client.register_plugin(HelperPlugin)
        test_client.register_plugin(UnifiedRegistryPlugin)
        
        # 3. 获取插件实例
        plugins = test_client.get_registered_plugins()
        helper_plugin = next((p for p in plugins if isinstance(p, HelperPlugin)), None)
        unified_plugin = next((p for p in plugins if isinstance(p, UnifiedRegistryPlugin)), None)
        
        assert helper_plugin is not None
        assert unified_plugin is not None
        
        # 4. 注册依赖其他插件的命令
        @command_registry.command("get_helper", description="获取辅助数据")
        def get_helper_command(event: BaseMessageEvent):
            # 这里应该能访问到 helper_plugin，但由于测试环境限制，我们简化处理
            execution_log.append("accessed_helper")
            return "Helper data retrieved"
        get_helper_command.__is_command__ = True
        
        # 5. 测试命令执行
        mock_event = EventFactory.create_private_message(message="/get_helper", user_id="test_user")
        
        # 模拟执行（由于真实的插件查找比较复杂，这里简化）
        with patch.object(unified_plugin, '_find_plugin_for_function', return_value=helper_plugin):
            with patch.object(unified_plugin, '_execute_function') as mock_execute:
                mock_execute.return_value = "Helper data retrieved"
                
                await unified_plugin.handle_message_event(mock_event)
                mock_execute.assert_called()
        
        # 6. 清理
        test_client.unregister_plugin(helper_plugin)
        test_client.unregister_plugin(unified_plugin)



import re
from typing import Literal, Optional, Dict, Any, Union, Type, TypeVar, List, Iterable
from ncatbot.core.event.message_segment.message_segment import MessageSegment, get_class_by_name, MessageTypeNotFoundErr
from ncatbot.utils import NcatBotError, status, get_log
from dataclasses import fields
from .message_segment import (
    Text,
    PlainText,
    Face,
    Image,
    Video,
    At,
    AtAll,
    Reply,
    Node,
    Forward,
)

T = TypeVar('T', bound=MessageSegment)
LOG = get_log("MessageArray")

class MessageOperationError(NcatBotError):
    logger = LOG

class MessageFormatError(NcatBotError):
    logger = LOG

def parse_cq_code_to_onebot11(cq_string: str) -> List[Dict[str, Union[str, Dict[str, str]]]]:
    """
    将 CQ 码字符串解析为 OneBot 11 规范的消息数组格式，包含转义字符处理
    
    Args:
        cq_string: 包含 CQ 码的字符串，例如 "[CQ:image,file=123.jpg]这是一段文本[CQ:face,id=123]"
        
    Returns:
        OneBot 11 规范的消息数组，例如:
        [
            {"type": "image", "data": {"file": "123.jpg"}},
            {"type": "text", "data": {"text": "这是一段文本"}},
            {"type": "face", "data": {"id": "123"}}
        ]
    """
    # 正则表达式匹配 CQ 码
    cq_pattern = re.compile(r'\[CQ:([^,\]]+)(?:,([^\]]+))?\]')
    
    # 初始化结果列表
    message_segments = []
    last_pos = 0
    
    # HTML 实体转义映射
    html_unescape_map = {
        '&amp;': '&',
        '&#91;': '[',
        '&#93;': ']',
        '&#44;': ','
    }
    
    def unescape_cq(text: str) -> str:
        """取消 CQ 码中的 HTML 实体转义"""
        for escaped, unescaped in html_unescape_map.items():
            text = text.replace(escaped, unescaped)
        return text
    
    # 遍历所有匹配的 CQ 码
    for match in cq_pattern.finditer(cq_string):
        # 处理 CQ 码之前的文本（如果有）
        text_before = cq_string[last_pos:match.start()]
        if text_before:
            # 普通文本也需要反转义
            message_segments.append({
                "type": "text",
                "data": {"text": unescape_cq(text_before)}
            })
        
        # 处理 CQ 码
        cq_type = match.group(1)
        cq_params_str = match.group(2) or ""
        
        # 解析 CQ 码参数
        params = {}
        for param in cq_params_str.split(','):
            if '=' in param:
                key, value = param.split('=', 1)
                # 对参数值进行反转义处理
                params[key] = unescape_cq(value)
        
        # 添加到结果列表
        message_segments.append({
            "type": cq_type,
            "data": params
        })
        
        # 更新最后位置
        last_pos = match.end()
    
    # 处理最后一个 CQ 码之后的文本（如果有）
    text_after = cq_string[last_pos:]
    if text_after:
        message_segments.append({
            "type": "text",
            "data": {"text": unescape_cq(text_after)}
        })
    
    return message_segments

def process_dict(data: dict) -> MessageSegment:
    msg_seg_type = data.get("type")
    msg_data = data.get("data")
    return get_class_by_name(msg_seg_type).from_dict(msg_data)

def process_iterable(data: list) -> list[MessageSegment]:
    result = []
    for item in data:
        result = result + process_item(item)
    return result

def process_item(item: Union[List[Union[dict, MessageSegment]], Union[dict, MessageSegment]]) -> list[MessageSegment]:
    """
    处理消息(单个或迭代器)，返回对应的 list[MessageSegment] 对象
    
    Args:
        item: 可以是 MessageSegment 对象或字典
        
    Returns:
        处理后的 MessageSegment 对象，如果无法处理则返回 None
    """
    while (isinstance(item, list) or isinstance(item, tuple)) and len(item) == 1:
        item = item[0]
        
    if isinstance(item, MessageSegment):
        return [item]
    if isinstance(item, str):
        # 字符串, 当 CQ 码处理
        item = parse_cq_code_to_onebot11(item)
    if isinstance(item, dict):
        # 字典, 当消息段处理
        return [process_dict(item)]    
    if isinstance(item, Iterable):
        return process_iterable(item)    
    return []

class MessageArray:
    """表示一条消息的数据结构
    支持字典构造和 MessageSegment 构造
    """
    messages: list[MessageSegment] = []
        
    @classmethod
    def from_list(cls, data: list[Union[dict, MessageSegment]]):
        """"其实也支持单个"""
        return cls(data)
    
    def to_list(self) -> list[Dict]:
        data = []
        for item in self.messages:
            data.append(item.to_dict())
        return data
    
    def __init__(self, *args):
        self.messages = process_item(args)
        if self.is_forward_msg():
            mlen = len(self.messages)
            if mlen != len(self.filter(Forward)) and mlen != len(self.filter(Node)):
                raise MessageFormatError("消息格式错误, 合并转发消息严禁和其它类型消息混用")
    
    def add_by_list(self, data: list[Union[dict, MessageSegment]]):
        self.messages.extend(process_item(data))
        return self
    
    def add_by_segment(self, segment: MessageSegment):
        self.messages.append(segment)
        return self
    
    def add_by_dict(self, data: dict):
        self.messages.append(process_dict(data))
        return self
    
    def add_text(self, text: str):
        self.messages.extend(process_item(text))
        return self
    
    def add_image(self, image: str):
        self.messages.append(Image(file=image))
        return self
    
    def add_at(self, user_id: Union[str, int]):
        self.messages.append(At(user_id))
        return self
    
    def add_at_all(self):
        self.messages.append(AtAll())
        return self
    
    def add_reply(self, message_id: Union[str, int]):
        self.messages.append(Reply(message_id))
        return self
    
    def __add__(self, other):
        messages = self.messages + process_item(other)
        return MessageArray(messages)
        
    def __radd__(self, other):
        return self.__add__(other)
    
    def is_forward_msg(self):
        return len(self.filter(Forward)) > 0 or len(self.filter(Node)) > 0
    
    async def plain_forward_msg(self) -> Forward: 
        """把转发id格式的消息展平为解析完毕的 Forward
        Raises:
            MessageTypeError: _description_

        Returns:
            list[Node]: _description_
        """
        if not self.is_forward_msg():
            raise MessageOperationError("此消息不是合并转发消息, 无法展平")
        msg = self.filter(Forward)
        if len(msg) == 0:
            return self.messages
        return await status.global_api.get_forward_msg(msg[0].id)
    
    def filter(self, cls: Union[Type[T], None] = None) -> List[T]:
        if cls is None:
            return self.messages
        msg = []
        if not issubclass(cls, MessageSegment):
            raise MessageTypeNotFoundErr(cls.__name__)
        for item in self.messages:
            # Text 和 PlainText 需要特殊处理
            if isinstance(item, cls) or (cls is Text and isinstance(item, PlainText)):
                msg.append(item)
        return msg
    
    def filter_text(self) -> List[Text]:
        return self.filter(Text)
    
    def filter_at(self) -> List[At]:
        return self.filter(At)
    
    def filter_image(self) -> List[Image]:
        return self.filter(Image)
    
    def filter_video(self) -> List[Video]:
        return self.filter(Video)
    
    def filter_face(self) -> List[Face]:
        return self.filter(Face)
    
    def is_user_at(self, user_id: Union[str, int], all_except: bool = False) -> bool:
        user_id = str(user_id)
        ats = self.filter(At)
        for at in ats:
            if at.qq == user_id:
                return True
        return not all_except and len(self.filter(AtAll)) > 0
    
    def __iter__(self):
        return self.messages.__iter__()
    
    def __len__(self):
        return len(self.messages)
    
    def __str__(self):
        return str(self.messages)
    
    def __repr__(self):
        return repr(self.messages)
    
if __name__ == "__main__":
    msg = MessageArray().filter(Text)



"""装饰器框架

提供直观的装饰器API，支持链式调用和参数验证。
"""

from typing import Callable, Any, List, Optional, Dict, Union, Type

from ..utils import CommandRegistrationError, OptionSpec, OptionGroupSpec, ParameterSpec

# TODO: 融合 option、param 以及函数签名自检测
# TODO: 支持类型系统

def check_is_command(func: Callable):
    if hasattr(func, "__is_command__"):
        raise CommandRegistrationError(
            func.__name__,
            "必须注册完参数后才能注册命令",
            details="必须注册完参数后才能注册命令",
            suggestions="将注册命令的装饰器移到最上方"
        )

def option(short_name: Optional[str] = None, 
          long_name: Optional[str] = None,
          help: str = ""):
    # TODO: 通过注解明确行为
    """选项装饰器
    
    用于添加命令选项（-v, --verbose等）。
    
    Args:
        short_name: 短选项名，如 "v"
        long_name: 长选项名，如 "verbose"
        help: 帮助文本
        default: 默认值
        group: 选项组ID
    
    Examples:
        @option("v", "verbose", help="详细输出")
    """
    def decorator(func: Callable) -> Callable:
        check_is_command(func)
        # 确保函数有选项列表属性
        if not hasattr(func, '__command_options__'):
            func.__command_options__ = []
        
        # 创建选项规格
        option_spec = OptionSpec(
            short_name=short_name,
            long_name=long_name,
            description=help
        )
        
        func.__command_options__.append(option_spec)
        return func
    
    return decorator

def option_group(
    choices: List[str],
    name: str,
    default: str,
    help: str = "",):
    # 用于指定基于字符串值选项
    # 仅 --xml, --yaml 格式等
    # TODO: 通过注解明确行为
    """选项组装饰器

    Args:
        choices (List[str]): 选项列表
        name: 选项组名称
        default (str): 默认选项
        help (str, optional): 帮助文本. Defaults to "".
    
    Examples:
        @exclusive_option_group(choices=["xml", "yaml"], default="xml", help="输出格式")
    """
    def decorator(func: Callable) -> Callable:
        # 确保函数有选项列表属性
        check_is_command(func)
        if not hasattr(func, '__command_option_groups__'):
            func.__command_option_groups__ = []
        
        # 创建选项规格
        option_spec = OptionGroupSpec(
            choices=choices,
            name=name,
            default=default,
            description=help
        )
        
        func.__command_option_groups__.append(option_spec)
        return func
    
    return decorator

def param(name: str,
         default: Any = None,
         required: Optional[bool] = False,
         help: str = "",
         choices: Optional[List[Any]] = None,
         validator: Optional[Callable] = None,
         examples: Optional[List[str]] = None,
         type_hints: Optional[Dict[Type, str]] = None,
         type_examples: Optional[Dict[Type, List[str]]] = None):
    """参数装饰器
    
    用于添加命名参数（--name=value）。
    
    Args:
        name: 参数名（不包含--前缀）
        default: 默认值
        required: 是否必需（None表示自动判断）
        help: 帮助文本
        choices: 可选值列表
        validator: 自定义验证器
        examples: 使用示例
        type_hints: 各类型的提示文本
        type_examples: 各类型的示例
    
    Examples:
        @param("env", type=str, choices=["dev", "test", "prod"])
        @param("port", type=int, default=8080)
        @param("input", type=[str, MessageSegment], 
               type_hints={str: "文件路径", MessageSegment: "图片文件"})
    """
    def decorator(func: Callable) -> Callable:
        # 确保函数有参数列表属性
        check_is_command(func)
            
        if not hasattr(func, '__command_params__'):
            func.__command_params__ = []
        
        # 创建参数规格
        param_spec = ParameterSpec(
            name=name,
            default=default,
            required=required,
            description=help,
            choices=choices,
            validator=validator,
            examples=examples or [],
            type_hints=type_hints or {},
            type_examples=type_examples or {},
            is_named=True,
            is_positional=False
        )
        
        func.__command_params__.append(param_spec)
        return func
    
    return decorator




"""函数分析器模块"""

from typing import Callable, List
from ncatbot.utils import get_log
from .sig_validator import SigValidator
from .param_validator import ParamsValidator
from ..utils import CommandSpec, FuncSpec

LOG = get_log(__name__)


def get_subclass_recursive(cls: type) -> List[type]:
    """递归获取类的所有子类
    
    Args:
        cls: 要获取子类的类
        
    Returns:
        List[type]: 包含该类及其所有子类的列表
    """
    return [cls] + [subcls for subcls in cls.__subclasses__() for subcls in get_subclass_recursive(subcls)]



class FuncAnalyser:
    """函数分析器
    
    分析函数签名，验证参数类型，并提供参数转换功能。
    支持的参数类型：str, int, float, bool, Sentence, MessageSegment 的子类。
    """
    
    def __init__(self, func: Callable):
        self.func_descriptor = FuncSpec(func)

    def analyze(self) -> CommandSpec:
        self.sig_validator = SigValidator(self.func_descriptor)
        self.actual_args_start_index, self.actual_params = self.sig_validator.analyze_signature()
        self.params_validator = ParamsValidator(self.func_descriptor, self.actual_params)
        return self.params_validator.analyze_params()
    
    
    


"""Constants used throughout the NcatBot CLI."""

# Package management
PYPI_SOURCE = "https://mirrors.aliyun.com/pypi/simple/"

# URLs
PLUGIN_INDEX_URL = (
    "https://raw.githubusercontent.com/ncatbot/NcatBot-Plugins/main/index.json"
)



from .commnad_mixin import CommandMixin
from typing import Any, List, Dict, Union
from ncatbot.core.event import BaseMessageEvent
from ncatbot.utils import PermissionGroup
from ncatbot.utils import get_log

LOG = get_log("ConfigMixin")

class Config:
    def __init__(self, data: dict):
        self.name = data['name']
        self.default_value = data['default_value']
        self.description = data['description']
        self.value_type = data['value_type']
        self.metadata = data['metadata']
    
    def __eq__(self, other: Any) -> bool:
        if isinstance(other, Config):
            return self.name == other.name
        if isinstance(other, str):
            return self.name == other
        return False

class ConfigMixin(CommandMixin):
    def _configurator(self, event: BaseMessageEvent, args: List[str]):
        # [config].[plugin_name] [config_name] [value]
        # TODO 错误提示
        try:
            if len(args) < 3:
                raise ValueError("参数不足")
            config_name = args[1]
            if config_name not in self._registered_configs:
                raise ValueError(f"配置 {config_name} 不存在")
            config = self._registered_configs[config_name]
            if isinstance(config.value_type, str):
                real_value = eval(config.value_type)(args[2])
            else:
                real_value = config.value_type(args[2])
            self.config[config_name] = real_value
            event.reply_sync(f"配置 {config_name} 设置为 {real_value} 成功")
        except Exception as e:
            LOG.error(f"插件 {self.name} 配置 {config_name} 设置失败: {e}")
            # event.reply_sync()
    
    def get_registered_configs(self) -> Dict[str, Config]:
        if not hasattr(self, '_registered_configs'):
            self._registered_configs: Dict[str, Config] = {}
        return self._registered_configs
    
    def register_config(self, name: str, default_value: Any, description: str = "", value_type: Union[str, type] = str, metadata: Dict[str, Any] = None):
        # TODO: 自动生成描述
        if not hasattr(self, "_registered_configs"):
            self._registered_configs: Dict[str, Config] = dict()
            self.register_command(f"config.{self.name}", self._configurator, permission=PermissionGroup.ADMIN.value)
        LOG.debug(f"插件 {self.name} 注册配置 {name}")
        if name not in self._registered_configs:
            if isinstance(value_type, str):
                value_type = eval(value_type)
            if self.config.get(name, None) is None:
                self.config[name] = value_type(default_value)
            self._registered_configs[name] = Config({
                'name': name,
                'default_value': default_value,
                'description': description,
                'value_type': value_type,
                'metadata': metadata
            })
            
        else:
            raise ValueError(f"配置 {name} 已存在")


"""触发器系统集成测试

测试触发器系统各组件的集成功能，包括完整触发流程、错误处理等。
"""

import pytest
import asyncio
from unittest.mock import Mock, patch

from ncatbot.plugin_system.builtin_plugin.unified_registry.trigger.preprocessor import MessagePreprocessor
from ncatbot.plugin_system.builtin_plugin.unified_registry.trigger.resolver import CommandResolver
from ncatbot.plugin_system.builtin_plugin.unified_registry.trigger.binder import ArgumentBinder
from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.lexer.tokenizer import (
    StringTokenizer, Token, TokenType
)
from ncatbot.utils.testing import EventFactory

class TestTriggerSystemIntegration:
    """触发器系统整体集成测试"""
    
    def test_complete_trigger_flow(self):
        """测试完整触发流程"""
        # 1. 创建各个组件
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=False
        )
        
        binder = ArgumentBinder()
        
        # 2. 准备命令注册
        mock_command = Mock()
        mock_command.func = lambda event, target: f"Banned {target}"
        
        commands = {("ban",): mock_command}
        resolver.build_index(commands, {})
        
        # 3. 创建测试事件
        event = EventFactory.create_group_message("/ban testuser")
        
        # 4. 执行完整流程
        
        # 步骤1：预处理
        preprocess_result = preprocessor.precheck(event)
        assert preprocess_result is not None
        assert preprocess_result.command_text == "ban testuser"
        
        # 步骤2：分词
        tokenizer = StringTokenizer(preprocess_result.command_text)
        tokens = tokenizer.tokenize()
        word_tokens = [t for t in tokens if t.type in (TokenType.WORD, TokenType.QUOTED_STRING)]
        assert len(word_tokens) >= 2  # ban, testuser
        
        # 步骤3：解析命令
        match = resolver.resolve_from_tokens(tokens)
        assert match is not None
        assert match.command is mock_command
        assert match.path_words == ("ban",)
        
        # 步骤4：参数绑定（模拟）
        mock_spec = Mock()
        mock_spec.get_kw_binding.return_value = {}
        
        # 模拟绑定结果
        from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.lexer.tokenizer import ParsedCommand
        
        mock_parsed = ParsedCommand(
            options={},
            named_params={},
            elements=[
                Mock(type="text", content="ban"),
                Mock(type="text", content="testuser")
            ],
            raw_tokens=[]
        )
        
        with pytest.MonkeyPatch().context() as m:
            mock_tokenizer = Mock()
            mock_tokenizer.parse_message.return_value = mock_parsed
            m.setattr("ncatbot.plugin_system.builtin_plugin.unified_registry.trigger.binder.MessageTokenizer", 
                     lambda: mock_tokenizer)
            
            bind_result = binder.bind(mock_spec, event, match.path_words, ["/"])
            
            assert bind_result.ok is True
            assert "testuser" in bind_result.args
    
    def test_trigger_flow_with_complex_command(self):
        """测试复杂命令的触发流程"""
        # 组件初始化
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!"],
            case_sensitive=False
        )
        
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=True
        )
        
        binder = ArgumentBinder()
        
        # 注册层级命令
        mock_command = Mock()
        mock_command.func = lambda event, user, reason, duration: f"Banned {user} for {reason} ({duration})"
        
        commands = {("admin", "user", "ban"): mock_command}
        aliases = {("admin", "u", "ban"): mock_command}
        resolver.build_index(commands, aliases)
        
        # 测试复杂命令: !admin u ban @user123 --reason=spam --duration=24h
        event = EventFactory.create_group_message("!admin u ban @user123 --reason=spam --duration=24h")
        
        # 流程执行
        
        # 1. 预处理
        preprocess_result = preprocessor.precheck(event)
        assert preprocess_result is not None
        assert preprocess_result.command_text == "admin u ban @user123 --reason=spam --duration=24h"
        
        # 2. 分词
        tokenizer = StringTokenizer(preprocess_result.command_text)
        tokens = tokenizer.tokenize()
        
        # 3. 解析命令
        match = resolver.resolve_from_tokens(tokens)
        assert match is not None
        assert match.command is mock_command
        assert match.path_words == ("admin", "u", "ban")
        
        # 4. 参数绑定（模拟复杂解析）
        mock_spec = Mock()
        mock_spec.get_kw_binding.return_value = {}
        
        from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.lexer.tokenizer import ParsedCommand
        
        mock_parsed = ParsedCommand(
            options={},
            named_params={"reason": "spam", "duration": "24h"},
            elements=[
                Mock(type="text", content="admin"),
                Mock(type="text", content="u"),
                Mock(type="text", content="ban"),
                Mock(type="text", content="@user123")
            ],
            raw_tokens=[]
        )
        
        with pytest.MonkeyPatch().context() as m:
            mock_tokenizer = Mock()
            mock_tokenizer.parse_message.return_value = mock_parsed
            m.setattr("ncatbot.plugin_system.builtin_plugin.unified_registry.trigger.binder.MessageTokenizer", 
                     lambda: mock_tokenizer)
            
            bind_result = binder.bind(mock_spec, event, match.path_words, ["!"])
            
            assert bind_result.ok is True
            assert "@user123" in bind_result.args
            assert "reason" in bind_result.named_args
            assert bind_result.named_args["reason"] == "spam"
            assert "duration" in bind_result.named_args
            assert bind_result.named_args["duration"] == "24h"
    
    def test_trigger_flow_no_prefix_match(self):
        """测试无前缀匹配的流程"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 无前缀的消息
        event = EventFactory.create_group_message("hello world")
        
        # 预处理应该失败
        preprocess_result = preprocessor.precheck(event)
        assert preprocess_result is None
    
    def test_trigger_flow_command_not_found(self):
        """测试命令未找到的流程"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=False
        )
        
        # 构建空索引
        resolver.build_index({}, {})
        
        event = EventFactory.create_group_message("/nonexistent command")
        
        # 预处理成功
        preprocess_result = preprocessor.precheck(event)
        assert preprocess_result is not None
        
        # 分词
        tokenizer = StringTokenizer(preprocess_result.command_text)
        tokens = tokenizer.tokenize()
        
        # 解析失败
        match = resolver.resolve_from_tokens(tokens)
        assert match is None
    
    def test_trigger_flow_with_options_and_quotes(self):
        """测试包含选项和引号的触发流程"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=False
        )
        
        binder = ArgumentBinder()
        
        # 注册命令
        mock_command = Mock()
        commands = {("deploy",): mock_command}
        resolver.build_index(commands, {})
        
        # 复杂命令: /deploy "my app" --env=production -v --force
        event = EventFactory.create_group_message('/deploy "my app" --env=production -v --force')
        
        # 执行流程
        preprocess_result = preprocessor.precheck(event)
        assert preprocess_result is not None
        
        tokenizer = StringTokenizer(preprocess_result.command_text)
        tokens = tokenizer.tokenize()
        
        # 验证分词结果
        word_tokens = [t for t in tokens if t.type == TokenType.WORD]
        quoted_tokens = [t for t in tokens if t.type == TokenType.QUOTED_STRING]
        option_tokens = [t for t in tokens if t.type in (TokenType.SHORT_OPTION, TokenType.LONG_OPTION)]
        
        assert len(word_tokens) >= 1  # deploy
        assert len(quoted_tokens) >= 1  # "my app"
        assert len(option_tokens) >= 3  # env, v, force
        
        match = resolver.resolve_from_tokens(tokens)
        assert match is not None
        assert match.command is mock_command


class TestTriggerErrorScenarios:
    """触发器错误场景测试"""
    
    def test_preprocessor_error_handling(self):
        """测试预处理器错误处理"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 创建有问题的事件
        event = Mock()
        event.message.messages = []  # 空消息
        
        result = preprocessor.precheck(event)
        assert result is None
        
        # 非文本首段
        event.message.messages = [Mock(msg_seg_type="image")]
        result = preprocessor.precheck(event)
        assert result is None
    
    def test_resolver_conflict_error(self):
        """测试解析器冲突错误"""
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=False  # 不允许层级
        )
        
        # 创建冲突的命令
        mock_cmd1 = Mock()
        mock_cmd2 = Mock()
        
        commands = {
            ("admin",): mock_cmd1,
            ("admin", "user"): mock_cmd2  # 与 admin 冲突
        }
        
        # 应该抛出冲突错误
        with pytest.raises(ValueError):
            resolver.build_index(commands, {})
    
    def test_binder_error_handling(self):
        """测试绑定器错误处理"""
        binder = ArgumentBinder()
        
        # 创建会出错的规格
        mock_spec = Mock()
        mock_spec.get_kw_binding.side_effect = Exception("Binding error")
        
        event = EventFactory.create_group_message("/test arg -v")
        
        # 应该抛出异常
        with pytest.raises(Exception):
            binder.bind(mock_spec, event, ("test",), ["/"])
    
    def test_tokenizer_error_integration(self):
        """测试分词器错误的集成处理"""
        from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.lexer.tokenizer import (
            QuoteMismatchError
        )
        
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 带有未匹配引号的消息
        event = EventFactory.create_group_message('/test "unmatched quote')
        
        preprocess_result = preprocessor.precheck(event)
        assert preprocess_result is not None
        
        # 分词应该抛出异常
        tokenizer = StringTokenizer(preprocess_result.command_text)
        with pytest.raises(QuoteMismatchError):
            tokenizer.tokenize()


class TestTriggerPerformanceIntegration:
    """触发器性能集成测试"""
    
    def test_large_command_set_performance(self):
        """测试大量命令集的性能"""
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=True
        )
        
        # 创建大量命令
        commands = {}
        for i in range(1000):
            mock_cmd = Mock()
            commands[(f"cmd{i}",)] = mock_cmd
        
        # 创建层级命令
        for i in range(100):
            mock_cmd = Mock()
            commands[("group", f"sub{i}")] = mock_cmd
        
        # 构建索引应该快速完成
        resolver.build_index(commands, {})
        
        # 解析应该快速完成
        tokens = [Token(TokenType.WORD, "cmd500", 0)]
        result = resolver.resolve_from_tokens(tokens)
        assert result is not None
        
        # 层级命令解析
        tokens = [
            Token(TokenType.WORD, "group", 0),
            Token(TokenType.WORD, "sub50", 6)
        ]
        result = resolver.resolve_from_tokens(tokens)
        assert result is not None
    
    def test_complex_message_parsing_performance(self):
        """测试复杂消息解析性能"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 创建复杂长消息
        complex_message = '/deploy app --env=production --config="very long config file path with spaces.json" --timeout=300 --retries=3 -v --force --debug'
        
        event = EventFactory.create_group_message(complex_message)
        
        # 预处理应该快速完成
        preprocess_result = preprocessor.precheck(event)
        assert preprocess_result is not None
        
        # 分词应该快速完成
        tokenizer = StringTokenizer(preprocess_result.command_text)
        tokens = tokenizer.tokenize()
        
        # 验证复杂解析结果
        assert len(tokens) > 10  # 应该有多个token


class TestTriggerStateManagement:
    """触发器状态管理测试"""
    
    def test_component_isolation(self):
        """测试组件隔离"""
        # 创建两套独立的组件
        preprocessor1 = MessagePreprocessor(require_prefix=True, prefixes=["/"], case_sensitive=False)
        preprocessor2 = MessagePreprocessor(require_prefix=True, prefixes=["!"], case_sensitive=True)
        
        resolver1 = CommandResolver(case_sensitive=False, allow_hierarchical=False)
        resolver2 = CommandResolver(case_sensitive=True, allow_hierarchical=True)
        
        # 在不同解析器中注册不同命令
        mock_cmd1 = Mock()
        mock_cmd2 = Mock()
        
        resolver1.build_index({("test1",): mock_cmd1}, {})
        resolver2.build_index({("test2",): mock_cmd2}, {})
        
        # 验证隔离
        tokens1 = [Token(TokenType.WORD, "test1", 0)]
        tokens2 = [Token(TokenType.WORD, "test2", 0)]
        
        assert resolver1.resolve_from_tokens(tokens1) is not None
        assert resolver1.resolve_from_tokens(tokens2) is None
        assert resolver2.resolve_from_tokens(tokens1) is None
        assert resolver2.resolve_from_tokens(tokens2) is not None
    
    def test_state_consistency(self):
        """测试状态一致性"""
        resolver = CommandResolver(case_sensitive=False, allow_hierarchical=False)
        
        mock_command = Mock()
        commands = {("test",): mock_command}
        resolver.build_index(commands, {})
        
        # 多次解析应该得到一致的结果
        tokens = [Token(TokenType.WORD, "test", 0)]
        
        result1 = resolver.resolve_from_tokens(tokens)
        result2 = resolver.resolve_from_tokens(tokens)
        
        assert result1.command is result2.command
        assert result1.path_words == result2.path_words
    
    def test_component_reinitialization(self):
        """测试组件重新初始化"""
        resolver = CommandResolver(case_sensitive=False, allow_hierarchical=False)
        
        # 第一次构建
        mock_cmd1 = Mock()
        resolver.build_index({("test1",): mock_cmd1}, {})
        
        tokens = [Token(TokenType.WORD, "test1", 0)]
        result = resolver.resolve_from_tokens(tokens)
        assert result is not None
        assert result.command is mock_cmd1
        
        # 重新构建（覆盖之前的索引）
        mock_cmd2 = Mock()
        resolver.build_index({("test2",): mock_cmd2}, {})
        
        # 旧命令应该不存在
        result = resolver.resolve_from_tokens(tokens)
        assert result is None
        
        # 新命令应该存在
        tokens = [Token(TokenType.WORD, "test2", 0)]
        result = resolver.resolve_from_tokens(tokens)
        assert result is not None
        assert result.command is mock_cmd2


class TestTriggerRealWorldScenarios:
    """触发器真实世界场景测试"""
    
    def test_admin_command_scenario(self):
        """测试管理员命令场景"""
        # 模拟真实的管理员命令处理流程
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!"],
            case_sensitive=False
        )
        
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=True
        )
        
        # 注册管理员命令
        ban_cmd = Mock()
        kick_cmd = Mock()
        mute_cmd = Mock()
        
        commands = {
            ("admin", "ban"): ban_cmd,
            ("admin", "kick"): kick_cmd,
            ("admin", "mute"): mute_cmd
        }
        
        aliases = {
            ("ban",): ban_cmd,  # 简化别名
            ("kick",): kick_cmd,
            ("mute",): mute_cmd
        }
        
        resolver.build_index(commands, aliases)
        
        # 测试不同的管理员命令
        test_commands = [
            "/admin ban @user123 --reason=spam",
            "!ban @user456",  # 使用别名
            "/admin mute @user789 --duration=1h",
            "/kick @user000"  # 别名
        ]
        
        for cmd_text in test_commands:
            event = EventFactory.create_group_message(cmd_text)
            
            preprocess_result = preprocessor.precheck(event)
            assert preprocess_result is not None
            
            tokenizer = StringTokenizer(preprocess_result.command_text)
            tokens = tokenizer.tokenize()
            
            match = resolver.resolve_from_tokens(tokens)
            assert match is not None
            assert match.command in [ban_cmd, kick_cmd, mute_cmd]
    
    def test_help_system_scenario(self):
        """测试帮助系统场景"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "?"],
            case_sensitive=False
        )
        
        resolver = CommandResolver(
            case_sensitive=False,
            allow_hierarchical=True
        )
        
        # 注册帮助命令
        help_cmd = Mock()
        
        commands = {
            ("help",): help_cmd
        }
        
        aliases = {
            ("h",): help_cmd,
            ("?",): help_cmd,
            ("帮助",): help_cmd  # 中文别名
        }
        
        resolver.build_index(commands, aliases)
        
        # 测试各种帮助调用方式
        help_commands = [
            "/help",
            "/h",
            "/?",
            "/帮助",
            "?help",  # 使用?前缀
        ]
        
        for cmd_text in help_commands:
            event = EventFactory.create_group_message(cmd_text)
            
            preprocess_result = preprocessor.precheck(event)
            if preprocess_result is None:
                continue  # 某些前缀可能不匹配
            
            tokenizer = StringTokenizer(preprocess_result.command_text)
            tokens = tokenizer.tokenize()
            
            match = resolver.resolve_from_tokens(tokens)
            if match is not None:  # 某些情况可能解析失败
                assert match.command is help_cmd



from ncatbot.core.event.event_data import BaseEventData
from typing import Literal, Optional

class Status:
    pass

class MetaEvent(BaseEventData):
    post_type: Literal['meta_event'] = None
    meta_event_type: Literal['heartbeat', 'lifecycle'] = None
    sub_type: Optional[Literal['enable', 'disable', 'connect']] = None # lifecycle
    interval: Optional[int] = None # heartbeat
    status: Optional[Status] = None # heartbeat
    def __init__(self, data: dict):
        self.meta_event_type = data.get("meta_event_type")
        self.sub_type = data.get("sub_type", None)
        self.interval = data.get("interval", None)
        self.status = data.get("status", None)
        super().__init__(data)


import re
import os
import base64
import urllib.request
from urllib.parse import urljoin
from ncatbot.utils import ncatbot_config

def convert_uploadable_object(i):
    """将可上传对象转换为标准格式"""

    def is_base64(s: str):
        if s.startswith("base64://"):
            return True
        if re.match(
            r"data:image/(jpg|jpeg|png|gif|bmp|webp|tiff|svg|mp4|avi|mov|wmv|flv|mkv|mpg|mpeg|m4v);base64,",
            s,
        ):
            return True
        return False

    def to_base64(s: str):
        if s.startswith("base64://"):
            return s
        if re.match(
            r"data:image/(jpg|jpeg|png|gif|bmp|webp|tiff|svg|mp4|avi|mov|wmv|flv|mkv|mpg|mpeg|m4v);base64,",
            s,
        ):
            m = re.match(
                r"data:image/(jpg|jpeg|png|gif|bmp|webp|tiff|svg|mp4|avi|mov|wmv|flv|mkv|mpg|mpeg|m4v);base64,(.*)",
                s,
            )
            return f"base64://{m.group(2)}"

    if i.startswith("http"):
        return i
    elif is_base64(i):
        return to_base64(i)
    elif os.path.exists(i):
        if ncatbot_config.is_napcat_local():
            return os.path.abspath(i)
        i = urljoin(
            "file:",
            f"base64://{base64.b64encode(open(i, 'rb').read()).decode('utf-8')}",
        )
        return i
    else:
        # 文件不存在时同样规范处理(可能在 NapCat 机子上)
        file_url = urljoin("file:", urllib.request.pathname2url(os.path.abspath(i)))
        return file_url


# -------------------------
# @Author       : Fish-LP fish.zh@outlook.com
# @Date         : 2025-02-12 13:41:02
# @LastEditors  : Fish-LP fish.zh@outlook.com
# @LastEditTime : 2025-03-15 16:14:50
# @Description  : 日志类
# @Copyright (c) 2025 by Fish-LP, Fcatbot使用许可协议
# -------------------------
import os
import re
import logging
import warnings
from typing import Optional, Dict, Pattern, Tuple, List, Union
from tqdm import tqdm as tqdm_original
from logging.handlers import TimedRotatingFileHandler

from ncatbot.utils.assets.color import Color



# 分配规则 - 支持输出到控制台和多目标匹配
rules = [
    ('database', 'db.log'),      # 数据库日志输出到文件
    ('network', 'network.log'),  # 同时输出到文件
]

# 日志格式配置
default_log_format = {
    "console": {
        "DEBUG": f"{Color.CYAN}[%(asctime)s.%(msecs)03d]{Color.RESET} "
        f"{Color.BLUE}%(colored_levelname)-8s{Color.RESET} "
        f"{Color.GRAY}[%(threadName)s|%(processName)s]{Color.RESET} "
        f"{Color.MAGENTA}%(name)s{Color.RESET} "
        f"{Color.YELLOW}%(filename)s:%(funcName)s:%(lineno)d{Color.RESET} "
        "| %(message)s",
        "INFO": f"{Color.CYAN}[%(asctime)s.%(msecs)03d]{Color.RESET} "
        f"{Color.GREEN}%(colored_levelname)-8s{Color.RESET} "
        f"{Color.MAGENTA}%(name)s{Color.RESET} ➜ "
        f"{Color.WHITE}%(message)s{Color.RESET}",
        "WARNING": f"{Color.CYAN}[%(asctime)s.%(msecs)03d]{Color.RESET} "
        f"{Color.YELLOW}%(colored_levelname)-8s{Color.RESET} "
        f"{Color.MAGENTA}%(name)s{Color.RESET} "
        f"{Color.RED}➜{Color.RESET} "
        f"{Color.YELLOW}%(message)s{Color.RESET}",
        "ERROR": f"{Color.CYAN}[%(asctime)s.%(msecs)03d]{Color.RESET} "
        f"{Color.RED}%(colored_levelname)-8s{Color.RESET} "
        f"{Color.GRAY}[%(filename)s]{Color.RESET}"
        f"{Color.MAGENTA}%(name)s:%(lineno)d{Color.RESET} "
        f"{Color.RED}➜{Color.RESET} "
        f"{Color.RED}%(message)s{Color.RESET}",
        "CRITICAL": f"{Color.CYAN}[%(asctime)s.%(msecs)03d]{Color.RESET} "
        f"{Color.BG_RED}{Color.WHITE}%(colored_levelname)-8s{Color.RESET} "
        f"{Color.GRAY}{{%(module)s}}{Color.RESET}"
        f"{Color.MAGENTA}[%(filename)s]{Color.RESET}"
        f"{Color.MAGENTA}%(name)s:%(lineno)d{Color.RESET} "
        f"{Color.BG_RED}➜{Color.RESET} "
        f"{Color.BOLD}%(message)s{Color.RESET}",
    },
    "file": {
        "DEBUG": "[%(asctime)s.%(msecs)03d] %(levelname)-8s [%(threadName)s|%(processName)s] %(name)s (%(filename)s:%(funcName)s:%(lineno)d) | %(message)s",
        "INFO": "[%(asctime)s.%(msecs)03d] %(levelname)-8s %(name)s ➜ %(message)s",
        "WARNING": "[%(asctime)s.%(msecs)03d] %(levelname)-8s %(name)s ➜ %(message)s",
        "ERROR": "[%(asctime)s.%(msecs)03d] %(levelname)-8s [%(filename)s]%(name)s:%(lineno)d ➜ %(message)s",
        "CRITICAL": "[%(asctime)s.%(msecs)03d] %(levelname)-8s {%(module)s}[%(filename)s]%(name)s:%(lineno)d ➜ %(message)s",
    },
}

# -------------------------------------------------
# 1. 自定义 tqdm
# -------------------------------------------------
class tqdm(tqdm_original):
    _STYLE_MAP = {
        "BLACK": Color.BLACK, "RED": Color.RED, "GREEN": Color.GREEN,
        "YELLOW": Color.YELLOW, "BLUE": Color.BLUE,
        "MAGENTA": Color.MAGENTA, "CYAN": Color.CYAN, "WHITE": Color.WHITE,
    }

    def __init__(self, *args, **kwargs):
        self._custom_colour = kwargs.pop("colour", "GREEN")
        kwargs.setdefault("bar_format",
                          f"{Color.CYAN}{{desc}}{Color.RESET} "
                          f"{Color.WHITE}{{percentage:3.0f}}%{Color.RESET} "
                          f"{Color.GRAY}[{{total}}/{{n_fmt}}]{Color.RESET}"
                          f"{Color.WHITE}|{{bar:20}}|{Color.RESET}"
                          f"{Color.BLUE}[{{elapsed}}]{Color.RESET}")
        kwargs.setdefault("ncols", 80)
        kwargs.setdefault("colour", None)
        super().__init__(*args, **kwargs)
        self.colour = self._custom_colour

    @property
    def colour(self):
        return self._colour

    @colour.setter
    def colour(self, color: str):
        color = (color or "GREEN").upper()
        valid_color = self._STYLE_MAP.get(color, Color.GREEN)
        self._colour = color
        if self.desc:
            self.desc = f"{valid_color}{self.desc}{Color.RESET}"

# -------------------------------------------------
# 2. 彩色日志
# -------------------------------------------------
LOG_LEVEL_TO_COLOR = {
    "DEBUG": Color.CYAN,
    "INFO": Color.GREEN,
    "WARNING": Color.YELLOW,
    "ERROR": Color.RED,
    "CRITICAL": Color.MAGENTA,
}

class ColoredFormatter(logging.Formatter):
    use_color = True

    def format(self, record: logging.LogRecord) -> str:
        try:
            if self.use_color:
                record.colored_levelname = (
                    f"{LOG_LEVEL_TO_COLOR.get(record.levelname, Color.RESET)}"
                    f"{record.levelname:8}{Color.RESET}"
                )
                record.colored_name = f"{Color.MAGENTA}{record.name}{Color.RESET}"
                record.colored_time = f"{Color.CYAN}{self.formatTime(record)}{Color.RESET}"
            else:
                record.colored_levelname = record.levelname
                record.colored_name = record.name
                record.colored_time = self.formatTime(record)
            return super().format(record)
        except Exception as e:
            warnings.warn(f"日志格式化错误: {e}")
            return f"[FORMAT ERROR] {record.getMessage()}"

# -------------------------------------------------
# 3. 自定义过滤器
# -------------------------------------------------
class RegisteredLoggerFilter(logging.Filter):
    """过滤器：只允许通过 get_log() 注册的 logger 的 DEBUG 信息通过控制台"""
    
    def filter(self, record: logging.LogRecord) -> bool:
        # 非 DEBUG 级别的日志都通过
        if record.levelno != logging.DEBUG:
            return True
        
        # 根 logger 的 DEBUG 信息总是通过
        if not record.name or record.name == "root":
            return True
        
        # DEBUG 级别的日志只有注册过的 logger 才通过
        try:
            from ncatbot.utils.status import status
            return status.is_registered_logger(record.name)
        except ImportError:
            # 如果状态模块未初始化，则默认通过
            return True

# -------------------------------------------------
# 4. 工具函数
# -------------------------------------------------
def _get_valid_log_level(level_name: str, default: str) -> str:
    level_name = level_name.upper()
    if hasattr(logging, level_name) and isinstance(getattr(logging, level_name), int):
        return level_name
    warnings.warn(f"Invalid log level: {level_name}, fallback to {default.upper()}")
    return default.upper()

def _compile_rules(raw_rules: List[Tuple[str, str]]) -> List[Tuple[Pattern, str]]:
    """
    编译规则列表，支持特殊值"console"
    :param raw_rules: 原始规则列表 [(pattern_str, target)]
    :return: 编译后的规则列表 [(compiled_regex, target)]
    """
    compiled_rules = []
    for pattern_str, target in raw_rules:
        try:
            compiled = re.compile(pattern_str)
            compiled_rules.append((compiled, target))
        except re.error as e:
            warnings.warn(f"Invalid regex pattern '{pattern_str}': {e}")
    return compiled_rules

# -------------------------------------------------
# 5. 全局初始化
# -------------------------------------------------
def setup_logging():
    """初始化全局日志系统"""
    # ---- 5.1 读取环境变量 ----
    console_level = _get_valid_log_level(os.getenv("LOG_LEVEL", "DEBUG"), "DEBUG")
    file_level = _get_valid_log_level(os.getenv("FILE_LOG_LEVEL", "DEBUG"), "DEBUG")
    log_dir = os.getenv("LOG_FILE_PATH", "./logs")
    backup_count = int(os.getenv("BACKUP_COUNT", "7"))
    os.makedirs(log_dir, exist_ok=True)

    # ---- 5.2 通用格式 ----
    console_fmt = os.getenv("LOG_FORMAT") or default_log_format["console"][console_level]
    file_fmt = os.getenv("LOG_FILE_FORMAT") or default_log_format["file"][file_level]

    # ---- 5.3 根记录器（控制台 + 主文件） ----
    root = logging.getLogger()
    root.setLevel(logging.DEBUG)
    root.handlers.clear()

    # 控制台处理器
    ch = logging.StreamHandler()
    ch.setLevel(console_level)
    ch.setFormatter(ColoredFormatter(console_fmt, datefmt="%H:%M:%S"))
    # 添加注册 logger 过滤器到控制台处理器
    ch.addFilter(RegisteredLoggerFilter())

    # 主文件处理器
    main_file = os.path.join(log_dir, "bot.log")
    fh_main = TimedRotatingFileHandler(
        main_file,
        when="midnight",
        interval=1,
        backupCount=backup_count,
        encoding="utf-8",
    )
    fh_main.suffix = "%Y_%m_%d"
    fh_main.extMatch = re.compile(r"^\d{4}_\d{2}_\d{2}$")
    fh_main.setLevel(file_level)
    fh_main.setFormatter(logging.Formatter(file_fmt))

    root.addHandler(ch)
    root.addHandler(fh_main)

    # ---- 5.4 重定向记录器 ----
    compiled_rules: List[Tuple[Pattern, str]] = _compile_rules(rules)
    
    # 创建过滤器类
    class RegexFilter(logging.Filter):
        def __init__(self, pattern: Pattern):
            super().__init__()
            self.pattern = pattern

        def filter(self, record: logging.LogRecord) -> bool:
            return bool(self.pattern.match(record.name))
    
    # 处理所有规则
    for pattern, target in compiled_rules:
        # 处理文件输出规则
        file_path = os.path.join(log_dir, target)
        file_handler = TimedRotatingFileHandler(
            file_path,
            when="midnight",
            interval=1,
            backupCount=backup_count,
            encoding="utf-8",
        )
        file_handler.suffix = "%Y_%m_%d"
        file_handler.extMatch = re.compile(r"^\d{4}_\d{2}_\d{2}$")
        file_handler.setLevel(file_level)
        file_handler.setFormatter(logging.Formatter(file_fmt))
        file_handler.addFilter(RegexFilter(pattern))
        # 添加到根记录器
        root.addHandler(file_handler)

    logging.getLogger("logger").debug("日志系统初始化")

# -------------------------------------------------
# 6. 获取 logger 的快捷函数
# -------------------------------------------------
def get_log(name: Optional[str] = None) -> logging.Logger:
    """返回一个logger；若name为空则返回root logger"""
    logger = logging.getLogger(name)
    # logger.setLevel(logging.INFO)
    
    # 注册非根 logger
    if name is not None:
        try:
            from ncatbot.utils.status import status
            status.register_logger(name)
        except ImportError:
            # 如果状态模块未初始化，则忽略注册
            pass
    
    return logger

# -------------------------------------------------
# 7. 启动时初始化一次
# -------------------------------------------------
setup_logging()

# -------------------------------------------------
# 8. 自测示例
# -------------------------------------------------
if __name__ == "__main__":
    import time
    from tqdm.contrib.logging import logging_redirect_tqdm

    root = get_log()
    db = get_log("database")
    net = get_log("network")
    test = get_log("test.module")

    root.info("启动测试")
    db.warning("数据库连接池 90% - 这条日志应该出现在db.log和bot.log")
    net.info("网络初始化 - 这条日志应该出现在控制台和network.log")
    net.error("连接超时 - 这条日志应该出现在控制台和network.log")
    test.info("测试模块日志 - 这条日志应该出现在bot.log")

    with logging_redirect_tqdm():
        for i in tqdm(range(50), desc="Task"):
            if i % 10 == 0:
                root.info(f"step {i}")
            time.sleep(0.05)

    root.info("测试完成")
    db.info("数据库关闭")
    net.info("网络关闭")
    
    # 打印所有处理器的信息用于调试
    print("\n日志处理器调试信息:")
    print(f"根记录器处理器数量: {len(root.handlers)}")
    for i, handler in enumerate(root.handlers):
        handler_type = type(handler).__name__
        if isinstance(handler, TimedRotatingFileHandler):
            print(f" 处理器 {i+1}: {handler_type} (文件: {handler.baseFilename})")
        elif isinstance(handler, logging.StreamHandler):
            print(f" 处理器 {i+1}: {handler_type} (控制台)")
        else:
            print(f" 处理器 {i+1}: {handler_type}")


"""消息预处理器测试

测试消息预处理器的功能，包括前缀检测、文本提取、大小写处理等。
"""

import pytest
from unittest.mock import Mock

from ncatbot.plugin_system.builtin_plugin.unified_registry.trigger.preprocessor import (
    MessagePreprocessor, PreprocessResult
)


class MockMessage:
    """模拟消息"""
    
    def __init__(self, messages):
        self.messages = messages


class MockTextMessage:
    """模拟文本消息"""
    
    def __init__(self, text: str):
        self.msg_seg_type = "text"
        self.text = text


class TestPreprocessResult:
    """预处理结果测试"""
    
    def test_preprocess_result_creation(self):
        """测试预处理结果创建"""
        result = PreprocessResult(command_text="hello world")
        assert result.command_text == "hello world"


class TestMessagePreprocessor:
    """消息预处理器测试"""
    
    def test_preprocessor_creation(self):
        """测试预处理器创建"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!"],
            case_sensitive=False
        )
        
        assert preprocessor.require_prefix is True
        assert preprocessor.prefixes == ["/", "!"]
        assert preprocessor.case_sensitive is False
    
    def test_preprocessor_creation_with_different_config(self):
        """测试不同配置的预处理器"""
        preprocessor = MessagePreprocessor(
            require_prefix=False,
            prefixes=["#", ">"],
            case_sensitive=True
        )
        
        assert preprocessor.require_prefix is False
        assert preprocessor.prefixes == ["#", ">"]
        assert preprocessor.case_sensitive is True
    
    def test_normalize_case_sensitive(self):
        """测试大小写敏感的规范化"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=True
        )
        
        result = preprocessor._normalize("Hello World")
        assert result == "Hello World"  # 保持原样
    
    def test_normalize_case_insensitive(self):
        """测试大小写不敏感的规范化"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        result = preprocessor._normalize("Hello World")
        assert result == "hello world"  # 转换为小写
    
    def test_precheck_empty_message(self):
        """测试空消息的预检查"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 空消息列表
        event = Mock()
        event.message.messages = []
        
        result = preprocessor.precheck(event)
        assert result is None
    
    def test_precheck_non_text_first_segment(self):
        """测试非文本首段的预检查"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 首段不是文本
        first_segment = Mock()
        first_segment.msg_seg_type = "image"
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        assert result is None
    
    def test_precheck_with_prefix_match(self):
        """测试前缀匹配的预检查"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!"],
            case_sensitive=False
        )
        
        # 创建带前缀的文本消息
        first_segment = MockTextMessage("/hello world")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert isinstance(result, PreprocessResult)
        assert result.command_text == "hello world"  # 前缀被移除
    
    def test_precheck_with_different_prefix(self):
        """测试不同前缀的匹配"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!", "#"],
            case_sensitive=False
        )
        
        test_cases = [
            ("!status", "status"),
            ("#help", "help"),
            ("/admin ban user", "admin ban user")
        ]
        
        for input_text, expected_output in test_cases:
            first_segment = MockTextMessage(input_text)
            event = Mock()
            event.message.messages = [first_segment]
            
            result = preprocessor.precheck(event)
            
            assert result is not None
            assert result.command_text == expected_output
    
    def test_precheck_no_prefix_match(self):
        """测试无前缀匹配的情况"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!"],
            case_sensitive=False
        )
        
        # 没有前缀的消息
        first_segment = MockTextMessage("hello world")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        assert result is None
    
    def test_precheck_prefix_case_insensitive(self):
        """测试大小写不敏感的前缀匹配"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!"],
            case_sensitive=False
        )
        
        # 大写前缀应该匹配
        first_segment = MockTextMessage("/HELLO WORLD")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert result.command_text == "HELLO WORLD"  # 保持原始大小写，只移除前缀
    
    def test_precheck_prefix_case_sensitive(self):
        """测试大小写敏感的前缀匹配"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=True
        )
        
        # 正确的前缀
        first_segment = MockTextMessage("/hello")
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        assert result is not None
        assert result.command_text == "hello"
        
        # 错误大小写的前缀（如果前缀本身是大小写敏感的）
        # 注意：这里假设前缀本身也区分大小写
        # 实际行为可能需要根据具体实现调整
    
    def test_precheck_no_prefix_required(self):
        """测试不要求前缀的情况"""
        preprocessor = MessagePreprocessor(
            require_prefix=False,
            prefixes=[],
            case_sensitive=False
        )
        
        # 任何文本都应该通过
        first_segment = MockTextMessage("any text here")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert result.command_text == "any text here"
    
    def test_precheck_whitespace_handling(self):
        """测试空白字符处理"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 前缀后有多个空格
        first_segment = MockTextMessage("/   hello world   ")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        # 前导空格应该被移除
        assert result.command_text == "hello world   "
    
    def test_precheck_empty_text_after_prefix(self):
        """测试前缀后为空的情况"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 只有前缀，没有内容
        first_segment = MockTextMessage("/")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert result.command_text == ""
    
    def test_precheck_prefix_whitespace_only_after(self):
        """测试前缀后只有空白字符的情况"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 前缀后只有空格
        first_segment = MockTextMessage("/   ")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert result.command_text == ""  # 空格被trim
    
    def test_precheck_multiple_matching_prefixes(self):
        """测试多个匹配前缀的情况"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["!", "!!"],  # 一个是另一个的前缀
            case_sensitive=False
        )
        
        # 应该匹配最先找到的
        first_segment = MockTextMessage("!!hello")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        # 根据实现，可能匹配 "!" 或 "!!"
        # 这里假设匹配第一个找到的
        assert "hello" in result.command_text
    
    def test_precheck_text_attribute_none(self):
        """测试text属性为None的情况"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # text属性为None
        first_segment = Mock()
        first_segment.msg_seg_type = "text"
        first_segment.text = None
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        # 应该处理None值
        assert result is None or result.command_text == ""


class TestMessagePreprocessorEdgeCases:
    """消息预处理器边界情况测试"""
    
    def test_very_long_prefix_list(self):
        """测试很长的前缀列表"""
        long_prefixes = [f"prefix{i}" for i in range(10)]
        
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=long_prefixes,
            case_sensitive=False
        )
        
        # 使用中间的某个前缀
        first_segment = MockTextMessage("prefix5test command")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert result.command_text == "test command"
    
    def test_unicode_prefix(self):
        """测试Unicode前缀"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["中", "→", "☆"],
            case_sensitive=False
        )
        
        # 使用Unicode前缀
        first_segment = MockTextMessage("中文命令 参数")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert result.command_text == "文命令 参数"
    
    def test_special_character_prefix(self):
        """测试特殊字符前缀"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["<", ">", "@", "#", "&"],
            case_sensitive=False
        )
        
        test_cases = [
            ("<command", "command"),
            (">execute", "execute"),
            ("@bot help", "bot help"),
            ("#tag", "tag"),
            ("&system", "system")
        ]
        
        for input_text, expected_output in test_cases:
            first_segment = MockTextMessage(input_text)
            event = Mock()
            event.message.messages = [first_segment]
            
            result = preprocessor.precheck(event)
            
            assert result is not None
            assert result.command_text == expected_output
    
    def test_empty_prefix_list(self):
        """测试空前缀列表"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=[],  # 空前缀列表
            case_sensitive=False
        )
        
        # 任何消息都不应该匹配
        first_segment = MockTextMessage("/hello")
        
        event = Mock()
        event.message.messages = [first_segment]
        
        result = preprocessor.precheck(event)
        
        assert result is None
    
    def test_none_prefix_list(self):
        """测试None前缀列表"""
        # 这种情况在构造函数中可能不被允许
        # 但如果允许，应该有合理的处理
        try:
            preprocessor = MessagePreprocessor(
                require_prefix=True,
                prefixes=None,
                case_sensitive=False
            )
            
            first_segment = MockTextMessage("/hello")
            event = Mock()
            event.message.messages = [first_segment]
            
            result = preprocessor.precheck(event)
            assert result is None
        except (TypeError, ValueError):
            # 如果构造函数拒绝None，也是可以接受的
            pass


class TestMessagePreprocessorIntegration:
    """消息预处理器集成测试"""
    
    def test_preprocessor_with_real_message_structure(self):
        """测试与真实消息结构的集成"""
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/", "!"],
            case_sensitive=False
        )
        
        # 模拟更真实的消息结构
        class RealTextSegment:
            def __init__(self, text):
                self.msg_seg_type = "text"
                self.text = text
        
        class RealMessage:
            def __init__(self, segments):
                self.messages = segments
        
        # 创建真实结构的消息
        text_segment = RealTextSegment("/admin user ban @someone")
        message = RealMessage([text_segment])
        
        event = Mock()
        event.message = message
        
        result = preprocessor.precheck(event)
        
        assert result is not None
        assert result.command_text == "admin user ban @someone"
    
    def test_preprocessor_workflow_integration(self):
        """测试预处理器工作流集成"""
        # 测试从消息接收到预处理结果的完整流程
        preprocessor = MessagePreprocessor(
            require_prefix=True,
            prefixes=["/"],
            case_sensitive=False
        )
        
        # 步骤1：接收消息
        raw_message = "/deploy myapp --env=production"
        
        # 步骤2：构建消息对象
        first_segment = MockTextMessage(raw_message)
        event = Mock()
        event.message.messages = [first_segment]
        
        # 步骤3：预处理
        result = preprocessor.precheck(event)
        
        # 步骤4：验证结果可用于后续处理
        assert result is not None
        assert result.command_text == "deploy myapp --env=production"
        
        # 结果应该可以传递给分词器等下游组件
        command_text = result.command_text
        assert "deploy" in command_text
        assert "myapp" in command_text
        assert "--env=production" in command_text



# -------------------------
# @Author       : Fish-LP fish.zh@outlook.com
# @Date         : 2025-05-15 18:39:14
# @LastEditors  : Fish-LP fish.zh@outlook.com
# @LastEditTime : 2025-05-15 18:42:44
# @Description  : 喵喵喵, 我还没想好怎么介绍文件喵
# @Copyright (c) 2025 by Fish-LP, Fcatbot使用许可协议 
# -------------------------
from .event import NcatBotEvent
from .event_bus import EventBus

__all__ = [
    'NcatBotEvent',
    'EventBus',
]


"""内置过滤器实现 v2.0"""

from typing import TYPE_CHECKING, Callable
from ncatbot.core import BaseMessageEvent
from ncatbot.utils import status
from ncatbot.utils.assets.literals import PermissionGroup
from .base import BaseFilter


if TYPE_CHECKING:
    from ncatbot.core.event import BaseMessageEvent

class GroupFilter(BaseFilter):
    """群聊消息过滤器"""
    
    def check(self, event: "BaseMessageEvent") -> bool:
        """检查是否为群聊消息"""
        return event.is_group_msg()

class PrivateFilter(BaseFilter):
    """私聊消息过滤器"""
    
    def check(self, event: "BaseMessageEvent") -> bool:
        """检查是否为私聊消息"""
        return not event.is_group_msg()

class AdminFilter(BaseFilter):
    """管理员权限过滤器"""
    
    def check(self, event: "BaseMessageEvent") -> bool:
        """检查用户是否有管理员权限"""
        if not status.global_access_manager:
            return False
        
        user_id = event.user_id
        return status.global_access_manager.user_has_role(user_id, PermissionGroup.ADMIN.value) \
            or status.global_access_manager.user_has_role(user_id, PermissionGroup.ROOT.value)

class RootFilter(BaseFilter):
    """Root权限过滤器"""
    
    def check(self, event: "BaseMessageEvent") -> bool:
        """检查用户是否有root权限"""
        if not status.global_access_manager:
            return False
        
        user_id = event.user_id
        return status.global_access_manager.user_has_role(user_id, PermissionGroup.ROOT.value)

class TrueFilter(BaseFilter):
    """True过滤器, 用于注册发送消息时调用的功能"""
    def check(self, event: "BaseMessageEvent") -> bool:
        return True

class CustomFilter(BaseFilter):
    """自定义函数过滤器包装器"""
    
    def __init__(self, filter_func: Callable[[BaseMessageEvent], bool], name: str = None):
        """初始化自定义过滤器
        
        Args:
            filter_func: 过滤器函数，签名为 (event: BaseMessageEvent) -> bool
            name: 过滤器名称
        """
        
        super().__init__(name or getattr(filter_func, '__name__', 'custom'))
        self.filter_func = filter_func
    
    def check(self, event: "BaseMessageEvent") -> bool:
        """执行自定义过滤器检查"""
        return self.filter_func(event)


# __init__.py

# Re-export all public symbols from color.py
from .color import Color

# Re-export all public symbols from literals.py
from .literals import (
    NAPCAT_WEBUI_SALT,
    WINDOWS_NAPCAT_DIR,
    LINUX_NAPCAT_DIR,
    INSTALL_SCRIPT_URL,
    NAPCAT_CLI_URL,
    PYPI_URL,
    NAPCAT_CLI_PATH,
    REQUEST_SUCCESS,
    OFFICIAL_GROUP_MESSAGE_EVENT,
    OFFICIAL_PRIVATE_MESSAGE_EVENT,
    OFFICIAL_REQUEST_EVENT,
    OFFICIAL_NOTICE_EVENT,
    OFFICIAL_STARTUP_EVENT,
    OFFICIAL_SHUTDOWN_EVENT,
    OFFICIAL_HEARTBEAT_EVENT,
    PLUGIN_BROKEN_MARK,
    STATUS_ONLINE,
    STATUS_Q_ME,
    STATUS_LEAVE,
    STATUS_BUSY,
    STATUS_DND,
    STATUS_HIDDEN,
    STATUS_LISTENING,
    STATUS_LOVE_YOU,
    STATUS_LEARNING,
    Status,
    PermissionGroup,
    DefaultPermission,
    EVENT_QUEUE_MAX_SIZE,
    PLUGINS_DIR,
    META_CONFIG_PATH,
    PERSISTENT_DIR,
)

__all__ = [
    # From color.py
    'Color',
    
    # From literals.py
    'NAPCAT_WEBUI_SALT',
    'WINDOWS_NAPCAT_DIR',
    'LINUX_NAPCAT_DIR',
    'INSTALL_SCRIPT_URL',
    'NAPCAT_CLI_URL',
    'PYPI_URL',
    'NAPCAT_CLI_PATH',
    'REQUEST_SUCCESS',
    'OFFICIAL_GROUP_MESSAGE_EVENT',
    'OFFICIAL_PRIVATE_MESSAGE_EVENT',
    'OFFICIAL_REQUEST_EVENT',
    'OFFICIAL_NOTICE_EVENT',
    'OFFICIAL_STARTUP_EVENT',
    'OFFICIAL_SHUTDOWN_EVENT',
    'OFFICIAL_HEARTBEAT_EVENT',
    'PLUGIN_BROKEN_MARK',
    'STATUS_ONLINE',
    'STATUS_Q_ME',
    'STATUS_LEAVE',
    'STATUS_BUSY',
    'STATUS_DND',
    'STATUS_HIDDEN',
    'STATUS_LISTENING',
    'STATUS_LOVE_YOU',
    'STATUS_LEARNING',
    'Status',
    'PermissionGroup',
    'DefaultPermission',
    'EVENT_QUEUE_MAX_SIZE',
    'PLUGINS_DIR',
    'META_CONFIG_PATH',
    'PERSISTENT_DIR',
]


from typing import Union, Literal, TYPE_CHECKING
from abc import abstractmethod, ABC
from ncatbot.core.event.event_data import MessageEventData
from ncatbot.utils import status
from ncatbot.core.event.sender import PrivateSender, GroupSender

if TYPE_CHECKING:
    from ncatbot.core.legacy import MessageChain


class BaseMessageEvent(MessageEventData, ABC):
    message_type: Literal["private", "group"] = None # 上级会获取
    sub_type: str = None # 下级会细化 Literal, 上级会获取
    
    def is_group_msg(self):
        return hasattr(self, "group_id")
    
    @abstractmethod
    async def reply(self, *args, **kwargs):
        pass
    
    @abstractmethod
    def reply_sync(self, *args, **kwargs):
        pass
    
    def get_core_properties_str(self):
        return super().get_core_properties_str() + [
            f"sender={self.sender}",
        ]

class AnonymousMessage(BaseMessageEvent):
    id: str = None
    name: str = None
    flag: str = None
    def __init__(self, data: dict):
        self.id = str(data.get("id"))
        self.name = data.get("name")
        self.flag = data.get("flag")
    

class GroupMessageEvent(BaseMessageEvent):
    message_type: Literal["group"] = None # 上级会获取
    anonymous: Union[None, AnonymousMessage]
    group_id: str = None
    sub_type: Literal["normal", "anonymous", "notice"] # 上级会获取
    sender: GroupSender = None
    
    def __init__(self, data: dict):
        super().__init__(data)
        self.sender = GroupSender(data.get("sender"))
        self.anonymous = AnonymousMessage(data.get("anonymous")) if data.get("anonymous", None) else None
        self.group_id = str(data.get("group_id"))
    
    def get_core_properties_str(self):
        return super().get_core_properties_str() + [
            f"group_id={self.group_id}"
        ]
    
    async def delete(self):
        return await status.global_api.delete_msg(self.message_id)
    
    def delete_sync(self):
        return status.global_api.delete_msg_sync(self.message_id)
    
    async def kick(self):
        return await status.global_api.set_group_kick(self.group_id, self.user_id)
    
    def kick_sync(self):
        return status.global_api.set_group_kick_sync(self.group_id, self.user_id)
    
    async def ban(self, ban_duration: int = 30):
        """禁言消息发送者(秒)
        """
        return await status.global_api.set_group_ban(self.group_id, self.user_id, ban_duration)
    
    def ban_sync(self, ban_duration: int = 30):
        return status.global_api.set_group_ban_sync(self.group_id, self.user_id, ban_duration)
    
    async def reply(self, text: str=None, image: str=None, at: bool=True, space: bool=True, rtf: "MessageChain"=None):
        return await status.global_api.post_group_msg(self.group_id, (" " if space else "") + text, self.user_id if at else None, reply=self.message_id, image=image, rtf=rtf)

    def reply_sync(self, text: str=None, image: str=None, at: bool=True, space: bool=True, rtf: "MessageChain"=None):
        return status.global_api.post_group_msg_sync(self.group_id, (" " if space else "") + text, self.user_id if at else None, reply=self.message_id, image=image, rtf=rtf)
    
class PrivateMessageEvent(BaseMessageEvent):
    message_type: Literal["private"] = None # 上级会获取
    sub_type: Literal["friend", "group", "other"] # 上级会获取
    sender: PrivateSender = None
    
    def __init__(self, data: dict):
        super().__init__(data)
        self.sender = PrivateSender(data.get("sender"))
    
    async def reply(self, text: str=None, image: str=None, rtf: "MessageChain"=None):
        return await status.global_api.post_private_msg(self.user_id, text, self.message_id, image, rtf)
    
    def reply_sync(self, text: str=None, image: str=None, rtf: "MessageChain"=None):
        return status.global_api.post_private_msg_sync(self.user_id, text, self.message_id, image, rtf)
    
    def __repr__(self):
        return super().__repr__()
    

    



from ncatbot.cli.commands.registry import registry
from ncatbot.cli.utils.colors import command, error, info, success
from ncatbot.utils import ncatbot_config as config


@registry.register(
    "setqq",
    "重新设置 QQ 号",
    "setqq",
    aliases=["qq"],
    category="sys",
    show_in_help=False,
)
def set_qq() -> str:
    """写入配置文件, 永久生效"""
    qq = input(info("请输入 QQ 号: "))
    if not qq.isdigit():
        print(error("QQ 号必须为数字!"))
        return set_qq()

    qq_confirm = input(info(f"请再输入一遍 QQ 号 {command(qq)} 并确认: "))
    if qq != qq_confirm:
        print(error("两次输入的 QQ 号不一致!"))
        return set_qq()

    config.save_permanent_config("bt_uin", qq)
    print(success(f"QQ 号已设置为 {qq}"))
    return qq



# -------------------------
# @Author       : Fish-LP fish.zh@outlook.com
# @Date         : 2025-03-21 18:06:59
# @LastEditors  : Fish-LP fish.zh@outlook.com
# @LastEditTime : 2025-08-04 15:49:02
# @Description  : 插件加载器
# @Copyright (c) 2025 by Fish-LP, Fcatbot使用许可协议 
# -------------------------
import asyncio
import importlib
# TODO 用 zipimport 实现 zip 格式插件
import sys
from collections import defaultdict, deque
from pathlib import Path
from types import ModuleType
from typing import Dict, Iterable, List, Optional, Set, Tuple, Type, Union, TYPE_CHECKING

from packaging.specifiers import SpecifierSet
from packaging.version import parse as parse_version
from logging import getLogger

from ncatbot.utils import ncatbot_config

from .decorator import CompatibleHandler
from .base_plugin import BasePlugin
from .event import EventBus
from .packhelper import PackageHelper
from .pluginsys_err import (
    PluginCircularDependencyError,
    PluginDependencyError,
    PluginVersionError,
    PluginNameConflictError,
)
from .config import config
from .rbac import RBACManager
from .builtin_plugin import SystemManager, UnifiedRegistryPlugin
from ncatbot.utils import get_log, status
if TYPE_CHECKING:
    import importlib.util

LOG = get_log("PluginLoader")
_PLUGINS_DIR = config.plugins_dir
_AUTO_INSTALL = config.auto_install_pip_pack


# ---------------------------------------------------------------------------
# 工具函数 / 小类
# ---------------------------------------------------------------------------
class _ModuleImporter:
    """把「目录->模块对象」的细节收敛到这里，方便做单元测试。"""

    def __init__(self, directory: str):
        self.directory = Path(directory).resolve()

    def load_all(self) -> Dict[str, ModuleType]:
        """返回 {插件名: 模块对象}。"""
        modules: Dict[str, ModuleType] = {}
        if not self.directory.exists():
            return modules

        original_path = [*sys.path]
        try:
            sys.path.insert(0, str(self.directory))
            for entry in self.directory.iterdir():
                if entry.is_dir() and (entry / "__init__.py").exists():
                    name, path = entry.name, entry
                elif entry.suffix == ".py":
                    name, path = entry.stem, entry
                else:
                    continue

                self._maybe_install_deps(path)
                modules[name] = self._import_single(name, path)
        finally:
            sys.path[:] = original_path
        return modules

    # ------------------------------------------------------------------
    # 私有
    # ------------------------------------------------------------------
    def _import_single(self, name: str, path: Path) -> ModuleType:
        try:
            if path.is_dir():
                return importlib.import_module(name)
            spec = importlib.util.spec_from_file_location(name, path)
            module = importlib.util.module_from_spec(spec)
            sys.modules[name] = module
            spec.loader.exec_module(module)
            LOG.info("成功导入插件文件: %s", name)
            return module
        except Exception as e:
            LOG.error("导入模块 %s 时出错: %s", name, e)
            raise

    def _maybe_install_deps(self, plugin_path: Path) -> None:
        if not _AUTO_INSTALL:
            return
        req_file = (
            plugin_path / "requirements.txt"
            if plugin_path.is_dir()
            else plugin_path.with_suffix(".requirements.txt")
        )
        if not req_file.exists():
            return

        for line in req_file.read_text(encoding="utf-8").splitlines():
            req = line.strip()
            if not req or req.startswith("#") or req.startswith("-"):
                continue
            self._ensure_package(req)

    def _ensure_package(self, req: str) -> None:
        """检查包是否存在，不存在则安装。"""
        PackageHelper.ensure(req)


class _DependencyResolver:
    """把「依赖图 -> 加载顺序」的逻辑独立出来，方便测试。"""

    def __init__(self) -> None:
        self._graph: Dict[str, Set[str]] = {}
        self._constraints: Dict[str, Dict[str, str]] = {}

    def build(self, plugin_classes: Iterable[Type[BasePlugin]]) -> None:
        self._graph.clear()
        self._constraints.clear()
        for cls in plugin_classes:
            self._graph[cls.name] = set(cls.dependencies.keys())
            self._constraints[cls.name] = cls.dependencies.copy()

    def resolve(self) -> List[str]:
        """返回按依赖排序后的插件名；出错抛异常。"""
        self._check_duplicate_names()
        in_degree = {k: 0 for k in self._graph}
        adj = defaultdict(list)
        for cur, deps in self._graph.items():
            for d in deps:
                adj[d].append(cur)
                in_degree[cur] += 1

        q = deque([k for k, v in in_degree.items() if v == 0])
        order = []
        while q:
            cur = q.popleft()
            order.append(cur)
            for nxt in adj[cur]:
                in_degree[nxt] -= 1
                if in_degree[nxt] == 0:
                    q.append(nxt)

        if len(order) != len(self._graph):
            raise PluginCircularDependencyError(set(self._graph) - set(order))
        return order

    # ------------------------------------------------------------------
    # 私有
    # ------------------------------------------------------------------
    def _check_duplicate_names(self) -> None:
        seen = set()
        for name in self._graph:
            if name in seen:
                raise PluginNameConflictError(name)
            seen.add(name)


# ---------------------------------------------------------------------------
# 主加载器
# ---------------------------------------------------------------------------
class PluginLoader:
    """插件加载器：负责插件的加载、卸载、重载、生命周期管理。"""

    def __init__(self, event_bus: EventBus, *, debug: bool = False) -> None:
        self.plugins: Dict[str, BasePlugin] = {}
        self.event_bus = event_bus or EventBus()
        self.rbac_manager = RBACManager(config.rbac_path)
        self._debug = debug
        self._resolver = _DependencyResolver()
        
        # 将rbac_manager注册到全局状态
        status.global_access_manager = self.rbac_manager

        if debug:
            LOG.warning("插件系统已切换为调试模式")

    # -------------------- 对外 API --------------------
    async def _init_plugin_in_thread(self, plugin: BasePlugin) -> None:
        """在插件的线程中初始化"""
        def _run_init():
            # 在线程中创建新的事件循环
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            try:
                # 运行初始化
                loop.run_until_complete(plugin.__onload__())
            finally:
                loop.close()
                
        # 在插件的线程池中执行初始化
        await asyncio.get_event_loop().run_in_executor(
            plugin.thread_pool, 
            _run_init
        )

    async def from_class_load_plugins(
        self, plugin_classes: List[Type[BasePlugin]], **kwargs
    ) -> None:
        """从「插件类对象」加载。"""
        valid_classes = [cls for cls in plugin_classes if self._is_valid(cls)]
        self._resolver.build(valid_classes)

        load_order = self._resolver.resolve()
        temp = {}
        init_tasks = []
        
        for name in load_order:
            cls = next(c for c in valid_classes if c.name == name)
            LOG.info("加载插件「%s」", name)
            plugin = cls(
                event_bus=self.event_bus,
                debug=self._debug,
                rbac_manager=self.rbac_manager,
                plugin_loader=self,
                **kwargs,
            )
            temp[name] = plugin
            # 收集初始化任务
            init_tasks.append(self._init_plugin_in_thread(plugin))

        self.plugins.update(temp)
        self._validate_versions()
        # 并发执行所有插件的初始化
        await asyncio.gather(*init_tasks)

    async def load_plugin(self, plugin_class: Type[BasePlugin], **kwargs) -> None:
        plugin = plugin_class(
            event_bus=self.event_bus,
            debug=self._debug,
            rbac_manager=self.rbac_manager,
            plugin_loader=self,
        )
        self.plugins[plugin.name] = plugin
        await self._init_plugin_in_thread(plugin)
        return plugin

    async def load_builtin_plugins(self) -> None:
        """加载内置插件。"""
        # 内置插件要在这里声明
        plugins = [SystemManager, UnifiedRegistryPlugin]
        for plugin in plugins:
            await self.load_plugin(plugin)
        LOG.info("已加载内置插件数 [%d]", len(self.plugins))

    async def load_plugins(self, plugins_path: str = _PLUGINS_DIR, **kwargs) -> None:
        """从目录批量加载。"""
        path = Path(plugins_path or _PLUGINS_DIR).resolve()
        await self.load_builtin_plugins()
        if not path.exists():
            LOG.info("插件目录: %s 不存在……跳过加载插件", path)
            return
        
        if ncatbot_config.plugin.skip_plugin_load:
            LOG.info("跳过外部插件加载")
            return

        LOG.info("从 %s 导入插件", path)
        importer = _ModuleImporter(str(path))
        modules = importer.load_all()

        plugin_classes: List[Type[BasePlugin]] = []
        for mod in modules.values():
            for cls_name in getattr(mod, "__all__", []):
                cls = getattr(mod, cls_name)
                if self._is_valid(cls):
                    plugin_classes.append(cls)

        await self.from_class_load_plugins(plugin_classes, **kwargs)
        LOG.info("已加载插件数 [%d]", len(self.plugins))
        # self._load_compatible_data()

    async def unload_plugin(self, name: str, **kwargs) -> bool:
        """卸载单个插件。"""
        plugin = self.plugins.get(name)
        if not plugin:
            LOG.warning("插件 '%s' 未加载，无法卸载", name)
            return False
        try:
            await plugin.__unload__(**kwargs)
            del self.plugins[name]
            return True
        except Exception as e:
            LOG.error("卸载插件 '%s' 时发生错误: %s", name, e)
            return False

    async def reload_plugin(self, name: str, **kwargs) -> bool:
        """重载单个插件。"""
        try:
            old = self.plugins.get(name)
            if old:
                # 调用重载前的钩子函数
                old._reinit_()
                await old.on_reload()
                if not await self.unload_plugin(name):
                    return False

            module_name = old.__class__.__module__ if old else self._guess_module(name)
            module = importlib.import_module(module_name)
            importlib.reload(module)

            cls = self._find_plugin_class_in_module(module, name)
            if not cls:
                LOG.error("在模块中未找到插件 '%s'", name)
                return False

            new = await self.load_plugin(cls)
            
            # 执行兼容处理（这玩意应该不需要了）
            # for _, func in _iter_callables(new):
            #     for handler in CompatibleHandler._subclasses:
            #         if handler.check(func):
            #             handler.handle(new, func, self.event_bus, new)
                        
            self.plugins[name] = new
            LOG.info("插件 '%s' 重载成功", name)
            return True
        except Exception as e:
            LOG.error("重载插件 '%s' 失败: %s", name, e)
            return False

    async def unload_all(self, **kwargs) -> None:
        """一键异步卸载全部插件。"""
        self.rbac_manager.save(config.rbac_path)
        await asyncio.gather(
            *(self.unload_plugin(name, **kwargs) for name in list(self.plugins))
        )

    # -------------------- 查询 API --------------------
    def get_plugin(self, name: str) -> Optional[BasePlugin]:
        """根据插件名称获取插件实例。

        Args:
            name: 插件名称。

        Returns:
            插件实例；若不存在则返回 None。
        """
        return self.plugins.get(name, None)

    def get_metadata(self, name: str) -> dict:
        return self.plugins[name].meta_data

    def list_plugins(self, *, obj: bool = False) -> List[Union[str, BasePlugin]]:
        """插件列表

        Args:
            obj: 实例模式

        Returns:
            插件实例/插件名称列表
        """
        return list(self.plugins.values()) if obj else list(self.plugins.keys())

    # -------------------- 私有辅助 --------------------
    @staticmethod
    def _is_valid(cls: Type[BasePlugin]) -> bool:
        return all(hasattr(cls, attr) for attr in ("name", "version", "dependencies"))

    def _validate_versions(self) -> None:
        """检查已加载插件的版本约束。"""
        for plugin_name, constraints in self._resolver._constraints.items():
            for dep_name, constraint in constraints.items():
                dep = self.plugins.get(dep_name)
                if not dep:
                    raise PluginDependencyError(plugin_name, dep_name, constraint)
                if not SpecifierSet(constraint).contains(parse_version(dep.version)):
                    raise PluginVersionError(
                        plugin_name, dep_name, constraint, dep.version
                    )

    # def _load_compatible_data(self) -> None:
    #     """运行兼容处理器。"""
    #     for plugin in self.plugins.values():
    #         for _, func in _iter_callables(plugin):
    #             for handler in CompatibleHandler._subclasses:
    #                 if handler.check(func):
    #                     # 传入plugin实例
    #                     handler.handle(plugin, func, self.event_bus)

    def _guess_module(self, plugin_name: str) -> str:
        """根据插件名猜模块名；简单实现，如有需要可扩展。"""
        for entry in Path(_PLUGINS_DIR).iterdir():
            if entry.name == plugin_name:
                return entry.stem
        raise ValueError(f"无法定位插件 {plugin_name} 的模块")

    def _find_plugin_class_in_module(
        self, module: ModuleType, plugin_name: str
    ) -> Optional[Type[BasePlugin]]:
        for obj in vars(module).values():
            if (
                isinstance(obj, type)
                and issubclass(obj, BasePlugin)
                and getattr(obj, "name", None) == plugin_name
            ):
                return obj
        return None


# ---------------------------------------------------------------------------
# 小工具
# ---------------------------------------------------------------------------
def _iter_callables(obj):
    """遍历对象的所有可调用成员。"""
    for attr in dir(obj):
        value = getattr(obj, attr)
        if callable(value):
            yield attr, value